"""
Test that stepping works even when the OS Plugin doesn't report
all threads at every stop.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestOSPluginStepping(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfWindows
    @skipIf(oslist=["freebsd"], bugnumber="llvm.org/pr48352")
    def test_python_os_plugin(self):
        """Test that stepping works when the OS Plugin doesn't report all
        threads at every stop"""
        self.build()
        self.main_file = lldb.SBFileSpec("main.cpp")
        self.run_python_os_step_missing_thread(False)

    @skipIfWindows
    @skipIf(oslist=["freebsd"], bugnumber="llvm.org/pr48352")
    def test_python_os_plugin_prune(self):
        """Test that pruning the unreported PlanStacks works"""
        self.build()
        self.main_file = lldb.SBFileSpec("main.cpp")
        self.run_python_os_step_missing_thread(True)

    def get_os_thread(self):
        return self.process.GetThreadByID(0x111111111)

    def is_os_thread(self, thread):
        id = thread.GetID()
        return id == 0x111111111

    def run_python_os_step_missing_thread(self, do_prune):
        """Test that the Python operating system plugin works correctly"""

        # Our OS plugin does NOT report all threads:
        result = self.dbg.HandleCommand(
            "settings set process.experimental.os-plugin-reports-all-threads false"
        )

        python_os_plugin_path = os.path.join(self.getSourceDir(), "operating_system.py")
        (target, self.process, thread, thread_bkpt) = lldbutil.run_to_source_breakpoint(
            self, "first stop in thread - do a step out", self.main_file
        )

        main_bkpt = target.BreakpointCreateBySourceRegex(
            "Stop here and do not make a memory thread for thread_1", self.main_file
        )
        self.assertEqual(
            main_bkpt.GetNumLocations(), 1, "Main breakpoint has one location"
        )

        # There should not be an os thread before we load the plugin:
        self.assertFalse(
            self.get_os_thread().IsValid(), "No OS thread before loading plugin"
        )

        # Now load the python OS plug-in which should update the thread list and we should have
        # an OS plug-in thread overlaying thread_1 with id 0x111111111
        command = (
            "settings set target.process.python-os-plugin-path '%s'"
            % python_os_plugin_path
        )
        self.dbg.HandleCommand(command)

        # Verify our OS plug-in threads showed up
        os_thread = self.get_os_thread()
        self.assertTrue(
            os_thread.IsValid(),
            "Make sure we added the thread 0x111111111 after we load the python OS plug-in",
        )

        # Now we are going to step-out.  This should get interrupted by main_bkpt.  We've
        # set up the OS plugin so at this stop, we have lost the OS thread 0x111111111.
        # Make sure both of these are true:
        os_thread.StepOut()

        stopped_threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, main_bkpt
        )
        self.assertEqual(len(stopped_threads), 1, "Stopped at main_bkpt")
        thread = self.process.GetThreadByID(0x111111111)
        self.assertFalse(thread.IsValid(), "No thread 0x111111111 on second stop.")

        # Make sure we still have the thread plans for this thread:
        # First, don't show unreported threads, that should fail:
        command = "thread plan list -t 0x111111111"
        result = lldb.SBCommandReturnObject()
        interp = self.dbg.GetCommandInterpreter()
        interp.HandleCommand(command, result)
        self.assertFalse(
            result.Succeeded(), "We found no plans for the unreported thread."
        )
        # Now do it again but with the -u flag:
        command = "thread plan list -u -t 0x111111111"
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand(command, result)
        self.assertTrue(result.Succeeded(), "We found plans for the unreported thread.")

        if do_prune:
            # Prune the thread plan and continue, and we will run to exit.
            interp.HandleCommand("thread plan prune 0x111111111", result)
            self.assertTrue(
                result.Succeeded(), "Found the plan for 0x111111111 and pruned it"
            )

            # List again, make sure it doesn't work:
            command = "thread plan list -u -t 0x111111111"
            interp.HandleCommand(command, result)
            self.assertFalse(
                result.Succeeded(), "We still found plans for the unreported thread."
            )

            self.process.Continue()
            self.assertState(self.process.GetState(), lldb.eStateExited, "We exited.")
        else:
            # Now we are going to continue, and when we hit the step-out breakpoint, we will
            # put the OS plugin thread back, lldb will recover its ThreadPlanStack, and
            # we will stop with a "step-out" reason.
            self.process.Continue()
            os_thread = self.get_os_thread()
            self.assertTrue(os_thread.IsValid(), "The OS thread is back after continue")
            self.assertIn(
                "step out", os_thread.GetStopDescription(100), "Completed step out plan"
            )



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


@run
def testLifecycleContextDestroy():
    ctx = Context()

    def callback(foo):
        ...

    handler = ctx.attach_diagnostic_handler(callback)
    assert handler.attached
    # If context is destroyed before the handler, it should auto-detach.
    ctx = None
    gc.collect()
    assert not handler.attached

    # And finally collecting the handler should be fine.
    handler = None
    gc.collect()


@run
def testLifecycleExplicitDetach():
    ctx = Context()

    def callback(foo):
        ...

    handler = ctx.attach_diagnostic_handler(callback)
    assert handler.attached
    handler.detach()
    assert not handler.attached


@run
def testLifecycleWith():
    ctx = Context()

    def callback(foo):
        ...

    with ctx.attach_diagnostic_handler(callback) as handler:
        assert handler.attached
    assert not handler.attached


@run
def testLifecycleWithAndExplicitDetach():
    ctx = Context()

    def callback(foo):
        ...

    with ctx.attach_diagnostic_handler(callback) as handler:
        assert handler.attached
        handler.detach()
    assert not handler.attached


# CHECK-LABEL: TEST: testDiagnosticCallback
@run
def testDiagnosticCallback():
    ctx = Context()

    def callback(d):
        # CHECK: DIAGNOSTIC: message='foobar', severity=DiagnosticSeverity.ERROR, loc=loc(unknown)
        print(
            f"DIAGNOSTIC: message='{d.message}', severity={d.severity}, loc={d.location}"
        )
        return True

    handler = ctx.attach_diagnostic_handler(callback)
    loc = Location.unknown(ctx)
    loc.emit_error("foobar")
    assert not handler.had_error


# CHECK-LABEL: TEST: testDiagnosticEmptyNotes
# TODO: Come up with a way to inject a diagnostic with notes from this API.
@run
def testDiagnosticEmptyNotes():
    ctx = Context()

    def callback(d):
        # CHECK: DIAGNOSTIC: notes=()
        print(f"DIAGNOSTIC: notes={d.notes}")
        return True

    handler = ctx.attach_diagnostic_handler(callback)
    loc = Location.unknown(ctx)
    loc.emit_error("foobar")
    assert not handler.had_error


# CHECK-LABEL: TEST: testDiagnosticNonEmptyNotes
@run
def testDiagnosticNonEmptyNotes():
    ctx = Context()
    ctx.emit_error_diagnostics = True

    def callback(d):
        # CHECK: DIAGNOSTIC:
        # CHECK:   message='arith.addi' op requires one result
        # CHECK:   notes=['see current operation: "arith.addi"() : () -> ()']
        print(f"DIAGNOSTIC:")
        print(f"  message={d.message}")
        print(f"  notes={list(map(str, d.notes))}")
        return True

    handler = ctx.attach_diagnostic_handler(callback)
    loc = Location.unknown(ctx)
    try:
        Operation.create("arith.addi", loc=loc).verify()
    except MLIRError:
        pass
    assert not handler.had_error


# CHECK-LABEL: TEST: testDiagnosticCallbackException
@run
def testDiagnosticCallbackException():
    ctx = Context()

    def callback(d):
        raise ValueError("Error in handler")

    handler = ctx.attach_diagnostic_handler(callback)
    loc = Location.unknown(ctx)
    loc.emit_error("foobar")
    assert handler.had_error


# CHECK-LABEL: TEST: testEscapingDiagnostic
@run
def testEscapingDiagnostic():
    ctx = Context()
    diags = []

    def callback(d):
        diags.append(d)
        return True

    handler = ctx.attach_diagnostic_handler(callback)
    loc = Location.unknown(ctx)
    loc.emit_error("foobar")
    assert not handler.had_error

    # CHECK: DIAGNOSTIC: <Invalid Diagnostic>
    print(f"DIAGNOSTIC: {str(diags[0])}")
    try:
        diags[0].severity
        raise RuntimeError("expected exception")
    except ValueError:
        pass
    try:
        diags[0].location
        raise RuntimeError("expected exception")
    except ValueError:
        pass
    try:
        diags[0].message
        raise RuntimeError("expected exception")
    except ValueError:
        pass
    try:
        diags[0].notes
        raise RuntimeError("expected exception")
    except ValueError:
        pass


# CHECK-LABEL: TEST: testDiagnosticReturnTrueHandles
@run
def testDiagnosticReturnTrueHandles():
    ctx = Context()

    def callback1(d):
        print(f"CALLBACK1: {d}")
        return True

    def callback2(d):
        print(f"CALLBACK2: {d}")
        return True

    ctx.attach_diagnostic_handler(callback1)
    ctx.attach_diagnostic_handler(callback2)
    loc = Location.unknown(ctx)
    # CHECK-NOT: CALLBACK1
    # CHECK: CALLBACK2: foobar
    # CHECK-NOT: CALLBACK1
    loc.emit_error("foobar")


# CHECK-LABEL: TEST: testDiagnosticReturnFalseDoesNotHandle
@run
def testDiagnosticReturnFalseDoesNotHandle():
    ctx = Context()

    def callback1(d):
        print(f"CALLBACK1: {d}")
        return True

    def callback2(d):
        print(f"CALLBACK2: {d}")
        return False

    ctx.attach_diagnostic_handler(callback1)
    ctx.attach_diagnostic_handler(callback2)
    loc = Location.unknown(ctx)
    # CHECK: CALLBACK2: foobar
    # CHECK: CALLBACK1: foobar
    loc.emit_error("foobar")



"""
Tests that C++ member and static variables have correct layout and scope.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    # We fail to lookup static members on Windows.
    @expectedFailureAll(oslist=["windows"])
    def test_access_from_main(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// stop in main", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr("my_a.m_a", result_type="short", result_value="1")
        self.expect_expr("my_a.s_b", result_type="long", result_value="2")
        self.expect_expr("my_a.s_c", result_type="int", result_value="3")

    # We fail to lookup static members on Windows.
    @expectedFailureAll(oslist=["windows"])
    def test_access_from_member_function(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// stop in member function", lldb.SBFileSpec("main.cpp")
        )
        self.expect_expr("m_a", result_type="short", result_value="1")
        self.expect_expr("s_b", result_type="long", result_value="2")
        self.expect_expr("s_c", result_type="int", result_value="3")

    # Currently lookups find variables that are in any scope.
    @expectedFailureAll()
    def test_access_without_scope(self):
        self.build()
        self.createTestTarget()
        self.expect(
            "expression s_c",
            error=True,
            startstr="error: use of undeclared identifier 's_d'",
        )

    # We fail to lookup static members on Windows.
    @expectedFailureAll(oslist=["windows"])
    def test_no_crash_in_IR_arithmetic(self):
        """
        Test that LLDB doesn't crash on evaluating specific expression involving
        pointer arithmetic and taking the address of a static class member.
        See https://bugs.llvm.org/show_bug.cgi?id=52449
        """
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// stop in main", lldb.SBFileSpec("main.cpp")
        )

        # This expression contains the following IR code:
        # ... i64 ptrtoint (i32* @_ZN1A3s_cE to i64)) ...
        expr = "(int*)100 + (long long)(&A::s_c)"

        # The IR interpreter doesn't support non-const operands to the
        # `GetElementPtr` IR instruction, so verify that it correctly fails to
        # evaluate expression.
        opts = lldb.SBExpressionOptions()
        opts.SetAllowJIT(False)
        value = self.target().EvaluateExpression(expr, opts)
        self.assertTrue(value.GetError().Fail())
        self.assertIn(
            "Can't evaluate the expression without a running target",
            value.GetError().GetCString(),
        )

        # Evaluating the expression via JIT should work fine.
        value = self.target().EvaluateExpression(expr)
        self.assertSuccess(value.GetError())

    def test_IR_interpreter_can_handle_getelementptr_constants_args(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// stop in main", lldb.SBFileSpec("main.cpp")
        )

        # This expression contains the following IR code:
        # ... getelementptr inbounds [2 x i32], [2 x i32]* %4, i64 0, i64 0
        expr = "arr[0]"

        # The IR interpreter supports const operands to the `GetElementPtr` IR
        # instruction, so it should be able to evaluate expression without JIT.
        opts = lldb.SBExpressionOptions()
        opts.SetAllowJIT(False)
        value = self.target().EvaluateExpression(expr, opts)
        self.assertSuccess(value.GetError())



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCPP20Standard(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler="clang", compiler_version=["<", "11.0"])
    def test_cpp20(self):
        """
        Tests that we can evaluate an expression in C++20 mode
        """
        self.build()
        lldbutil.run_to_source_breakpoint(self, "Foo{}", lldb.SBFileSpec("main.cpp"))

        self.expect(
            "expr -l c++11 -- Foo{} <=> Foo{}",
            error=True,
            substrs=[
                "'<=>' is a single token in C++20; add a space to avoid a change in behavior"
            ],
        )

        self.expect("expr -l c++20 -- Foo{} <=> Foo{}", substrs=["(bool) $0 = true"])



"""Test the 'step target' feature."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestStepTarget(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"
        self.end_line = line_number(self.main_source, "All done")

    @add_test_categories(["pyapi"])
    def get_to_start(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        self.main_source_spec = lldb.SBFileSpec(self.main_source)

        break_in_main = target.BreakpointCreateBySourceRegex(
            "Break here to try targetted stepping", self.main_source_spec
        )
        self.assertTrue(break_in_main, VALID_BREAKPOINT)
        self.assertGreater(break_in_main.GetNumLocations(), 0, "Has locations.")

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_in_main)

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        thread = threads[0]
        return thread

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr32343")
    def test_with_end_line(self):
        """Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""

        thread = self.get_to_start()

        error = lldb.SBError()
        thread.StepInto("lotsOfArgs", self.end_line, error)
        frame = thread.frames[0]

        self.assertEqual(frame.name, "lotsOfArgs", "Stepped to lotsOfArgs.")

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr32343")
    def test_with_end_line_bad_name(self):
        """Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""

        thread = self.get_to_start()

        error = lldb.SBError()
        thread.StepInto("lotsOfArgssss", self.end_line, error)
        frame = thread.frames[0]
        self.assertEqual(
            frame.line_entry.line, self.end_line, "Stepped to the block end."
        )

    def test_with_end_line_deeper(self):
        """Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""

        thread = self.get_to_start()

        error = lldb.SBError()
        thread.StepInto("modifyInt", self.end_line, error)
        frame = thread.frames[0]
        self.assertEqual(frame.name, "modifyInt", "Stepped to modifyInt.")

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr32343")
    def test_with_command_and_block(self):
        """Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""

        thread = self.get_to_start()

        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            'thread step-in -t "lotsOfArgs" -e block', result
        )
        self.assertTrue(result.Succeeded(), "thread step-in command succeeded.")

        frame = thread.frames[0]
        self.assertEqual(frame.name, "lotsOfArgs", "Stepped to lotsOfArgs.")

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr32343")
    def test_with_command_and_block_and_bad_name(self):
        """Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""

        thread = self.get_to_start()

        result = lldb.SBCommandReturnObject()
        self.dbg.GetCommandInterpreter().HandleCommand(
            'thread step-in -t "lotsOfArgsssss" -e block', result
        )
        self.assertTrue(result.Succeeded(), "thread step-in command succeeded.")

        frame = thread.frames[0]

        self.assertEqual(frame.name, "main", "Stepped back out to main.")
        # end_line is set to the line after the containing block.  Check that
        # we got there:
        self.assertEqual(frame.line_entry.line, self.end_line, "Got out of the block")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import func
from mlir.dialects import linalg
from mlir.dialects import tensor

from mlir.dialects.linalg.opdsl.lang import *

T1 = TV.T1
T2 = TV.T2


@linalg_structured_op
def matmul_mono(
    A=TensorDef(T, S.M, S.K),
    B=TensorDef(T, S.K, S.N),
    C=TensorDef(T, S.M, S.N, output=True),
):
    domain(D.m, D.n, D.k)
    C[D.m, D.n] += A[D.m, D.k] * B[D.k, D.n]


@linalg_structured_op
def matmul_poly(
    A=TensorDef(T1, S.M, S.K),
    B=TensorDef(T2, S.K, S.N),
    C=TensorDef(U, S.M, S.N, output=True),
    cast=TypeFnAttrDef(default=TypeFn.cast_signed),
):
    domain(D.m, D.n, D.k)
    C[D.m, D.n] += cast(U, A[D.m, D.k]) * cast(U, B[D.k, D.n])


with Context() as ctx, Location.unknown():
    module = Module.create()
    f16 = F16Type.get()
    f32 = F32Type.get()
    f64 = F64Type.get()
    i8 = IntegerType.get_signless(8)
    i16 = IntegerType.get_signless(16)
    i32 = IntegerType.get_signless(32)
    with InsertionPoint(module.body):

        # Multiplication indexing maps. We verify only the indexing maps of the
        # first multiplication and then do additional tests on casting and body
        # generation behavior.
        # CHECK: #[[$MUL_MAP_A:.+]] = affine_map<(d0, d1, d2) -> (d0, d2)>
        # CHECK: #[[$MUL_MAP_B:.+]] = affine_map<(d0, d1, d2) -> (d2, d1)>
        # CHECK: #[[$MUL_MAP_C:.+]] = affine_map<(d0, d1, d2) -> (d0, d1)>

        # CHECK-LABEL: func @test_matmul_mono
        # CHECK-SAME:  %[[A:.+]]: tensor<4x16xf32>
        # CHECK-SAME:  %[[B:.+]]: tensor<16x8xf32>
        # CHECK: %[[INITC:.+]] = tensor.empty() : tensor<4x8xf32>
        # CHECK: linalg.generic
        # CHECK-SAME: indexing_maps = [#[[$MUL_MAP_A]], #[[$MUL_MAP_B]], #[[$MUL_MAP_C]]]
        # CHECK-SAME: iterator_types = ["parallel", "parallel", "reduction"]
        # CHECK-SAME: ins(%[[A]], %[[B]]
        # CHECK-SAME: outs(%[[INITC]]
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f32), RankedTensorType.get((16, 8), f32)
        )
        def test_matmul_mono(lhs, rhs):
            init_result = tensor.EmptyOp([4, 8], f32)
            return matmul_mono(lhs, rhs, outs=[init_result.result])

        # CHECK-LABEL: @test_i8i8i32_matmul
        # CHECK:      ^{{.*}}(%[[A_ARG:.+]]: i8, %[[B_ARG:.+]]: i8, %[[C_ARG:.+]]: i32)
        # CHECK-NEXT:   %[[A_CAST:.+]] = arith.extsi %[[A_ARG]] : i8 to i32
        # CHECK-NEXT:   %[[B_CAST:.+]] = arith.extsi %[[B_ARG]] : i8 to i32
        # CHECK-NEXT:   %[[MUL:.+]] = arith.muli %[[A_CAST]], %[[B_CAST]] : i32
        # CHECK-NEXT:   %[[ADD:.+]] = arith.addi %[[C_ARG]], %[[MUL]] : i32
        # CHECK-NEXT:   linalg.yield %[[ADD]] : i32
        # CHECK-NEXT: -> tensor<4x8xi32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), i8),
            RankedTensorType.get((16, 8), i8),
            RankedTensorType.get((4, 8), i32),
        )
        def test_i8i8i32_matmul(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result])

        # CHECK-LABEL: @test_i8i8i32_matmul_unsigned
        # CHECK:   = arith.extui
        # CHECK:   = arith.extui
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), i8),
            RankedTensorType.get((16, 8), i8),
            RankedTensorType.get((4, 8), i32),
        )
        def test_i8i8i32_matmul_unsigned(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result], cast=TypeFn.cast_unsigned)

        # CHECK-LABEL: @test_i8i16i32_matmul
        # CHECK:      ^{{.*}}(%[[A_ARG:.+]]: i8, %[[B_ARG:.+]]: i16, %[[C_ARG:.+]]: i32)
        # CHECK-NEXT:   %[[A_CAST:.+]] = arith.extsi %[[A_ARG]] : i8 to i32
        # CHECK-NEXT:   %[[B_CAST:.+]] = arith.extsi %[[B_ARG]] : i16 to i32
        # CHECK-NEXT:   %[[MUL:.+]] = arith.muli %[[A_CAST]], %[[B_CAST]] : i32
        # CHECK-NEXT:   %[[ADD:.+]] = arith.addi %[[C_ARG]], %[[MUL]] : i32
        # CHECK-NEXT:   linalg.yield %[[ADD]] : i32
        # CHECK-NEXT: -> tensor<4x8xi32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), i8),
            RankedTensorType.get((16, 8), i16),
            RankedTensorType.get((4, 8), i32),
        )
        def test_i8i16i32_matmul(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result])

        # CHECK-LABEL: @test_i32i32i16_matmul
        # CHECK:      ^{{.*}}(%[[A_ARG:.+]]: i32, %[[B_ARG:.+]]: i32, %[[C_ARG:.+]]: i16)
        # CHECK-NEXT:   %[[A_CAST:.+]] = arith.trunci %[[A_ARG]] : i32 to i16
        # CHECK-NEXT:   %[[B_CAST:.+]] = arith.trunci %[[B_ARG]] : i32 to i16
        # CHECK-NEXT:   %[[MUL:.+]] = arith.muli %[[A_CAST]], %[[B_CAST]] : i16
        # CHECK-NEXT:   %[[ADD:.+]] = arith.addi %[[C_ARG]], %[[MUL]] : i16
        # CHECK-NEXT:   linalg.yield %[[ADD]] : i16
        # CHECK-NEXT: -> tensor<4x8xi16>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), i32),
            RankedTensorType.get((16, 8), i32),
            RankedTensorType.get((4, 8), i16),
        )
        def test_i32i32i16_matmul(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result])

        # CHECK-LABEL: @test_i8i8f32_matmul
        # CHECK:      ^{{.*}}(%[[A_ARG:.+]]: i8, %[[B_ARG:.+]]: i8, %[[C_ARG:.+]]: f32)
        # CHECK-NEXT:   %[[A_CAST:.+]] = arith.sitofp %[[A_ARG]] : i8 to f32
        # CHECK-NEXT:   %[[B_CAST:.+]] = arith.sitofp %[[B_ARG]] : i8 to f32
        # CHECK-NEXT:   %[[MUL:.+]] = arith.mulf %[[A_CAST]], %[[B_CAST]] : f32
        # CHECK-NEXT:   %[[ADD:.+]] = arith.addf %[[C_ARG]], %[[MUL]] : f32
        # CHECK-NEXT:   linalg.yield %[[ADD]] : f32
        # CHECK-NEXT: -> tensor<4x8xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), i8),
            RankedTensorType.get((16, 8), i8),
            RankedTensorType.get((4, 8), f32),
        )
        def test_i8i8f32_matmul(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result])

        # CHECK-LABEL: @test_i8i8f32_matmul_unsigned
        # CHECK:   = arith.uitofp
        # CHECK:   = arith.uitofp
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), i8),
            RankedTensorType.get((16, 8), i8),
            RankedTensorType.get((4, 8), f32),
        )
        def test_i8i8f32_matmul_unsigned(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result], cast=TypeFn.cast_unsigned)

        # CHECK-LABEL: @test_f16f16f32_matmul
        # CHECK:      ^{{.*}}(%[[A_ARG:.+]]: f16, %[[B_ARG:.+]]: f16, %[[C_ARG:.+]]: f32)
        # CHECK-NEXT:   %[[A_CAST:.+]] = arith.extf %[[A_ARG]] : f16 to f32
        # CHECK-NEXT:   %[[B_CAST:.+]] = arith.extf %[[B_ARG]] : f16 to f32
        # CHECK-NEXT:   %[[MUL:.+]] = arith.mulf %[[A_CAST]], %[[B_CAST]] : f32
        # CHECK-NEXT:   %[[ADD:.+]] = arith.addf %[[C_ARG]], %[[MUL]] : f32
        # CHECK-NEXT:   linalg.yield %[[ADD]] : f32
        # CHECK-NEXT: -> tensor<4x8xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f16),
            RankedTensorType.get((16, 8), f16),
            RankedTensorType.get((4, 8), f32),
        )
        def test_f16f16f32_matmul(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result])

        # CHECK-LABEL: @test_f64f64f32_matmul
        # CHECK:      ^{{.*}}(%[[A_ARG:.+]]: f64, %[[B_ARG:.+]]: f64, %[[C_ARG:.+]]: f32)
        # CHECK-NEXT:   %[[A_CAST:.+]] = arith.truncf %[[A_ARG]] : f64 to f32
        # CHECK-NEXT:   %[[B_CAST:.+]] = arith.truncf %[[B_ARG]] : f64 to f32
        # CHECK-NEXT:   %[[MUL:.+]] = arith.mulf %[[A_CAST]], %[[B_CAST]] : f32
        # CHECK-NEXT:   %[[ADD:.+]] = arith.addf %[[C_ARG]], %[[MUL]] : f32
        # CHECK-NEXT:   linalg.yield %[[ADD]] : f32
        # CHECK-NEXT: -> tensor<4x8xf32>
        @func.FuncOp.from_py_func(
            RankedTensorType.get((4, 16), f64),
            RankedTensorType.get((16, 8), f64),
            RankedTensorType.get((4, 8), f32),
        )
        def test_f64f64f32_matmul(lhs, rhs, init_result):
            return matmul_poly(lhs, rhs, outs=[init_result])


print(module)



# TestSwiftTypeLookup.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test the ability to look for type definitions at the command line
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftTypeLookup(TestBase):
    @swiftTest
    def test_swift_type_lookup(self):
        """Test the ability to look for type definitions at the command line"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        # for now, type lookup won't load the AST context, so force load it
        # before testing
        self.runCmd("expr 1")

        # check that basic cases work
        self.expect(
            "type lookup String",
            substrs=[
                '(struct Swift.String)',
                'struct String {',
                'extension Swift.String : '])
        self.expect(
            "type lookup Cla1",
            substrs=[
                '(class a.Cla1)',
                'class Cla1 {',
                'func bat(_ x: Swift.Int, y: a.Str1) -> Swift.Int'])
        self.expect(
            "type lookup Str1",
            substrs=[
                '(struct a.Str1)',
                'struct Str1 {',
                'func bar()',
                'var b'])

        # Regression test. Ensure "<could not resolve type>" is not output.
        self.expect(
            "type lookup String",
            matching=False,
            substrs=["<could not resolve type>"])

        # check that specifiers are honored
        # self.expect('type lookup class Cla1', substrs=['class Cla1 {'])
        # self.expect('type lookup struct Cla1', substrs=['class Cla1 {'], matching=False, error=True)

        # check that modules are honored
        self.expect("type lookup Swift.String",
                    substrs=['(struct Swift.String)',
                             'struct String {'])
        self.expect(
            "type lookup a.String",
            substrs=['(struct Swift.String)',
                     'struct String {'],
            matching=False)

        # check that a combination of module and specifier is honored
        # self.expect('type lookup class a.Cla1', substrs=['class Cla1 {'])
        # self.expect('type lookup class Swift.Cla1', substrs=['class Cla1 {'], matching=False, error=True)
        # self.expect('type lookup struct a.Cla1', substrs=['class Cla1 {'], matching=False, error=True)
        # self.expect('type lookup struct Swift.Cla1', substrs=['class Cla1 {'], matching=False, error=True)

        # check that nested types are looked up correctly
        self.expect('type lookup Toplevel.Nested.Deeper',
                    substrs=['class a.Toplevel.Nested.Deeper',
                             'struct a.Toplevel.Nested',
                             'class a.Toplevel',
                             'class Deeper', 
                             'func foo'])

        # check that mangled name lookup works
        self.expect(
            'type lookup _$sSiD',
            substrs=[
                'struct Int',
                'extension Swift.Int'])

        # check that we can look for generic things
        self.expect('type lookup Generic', substrs=['class Generic', 'foo'])
        self.expect(
            'type lookup Generic<String>',
            substrs=[
                'bound_generic_class a.Generic',
                'struct Swift.String',
                'class Generic',
                'func foo'])

        # check that we print comment text (and let you get rid of it)
        self.expect(
            'type lookup Int',
            substrs=['Creates a new instance'],
            matching=False)
        self.expect(
            'type lookup --show-help -- Int',
            substrs=['Creates a new instance'],
            matching=True)
        self.expect(
            'type lookup foo',
            substrs=[
                'func foo',
                'Int',
                'Double'],
            matching=True,
            ordered=False)
        self.expect(
            'type lookup --show-help -- print',
            substrs=[
                '/// ',
                'func print'],
            matching=True)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftExtraClangFlags(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(oslist=['windows'])
    @swiftTest
    def test_sanity(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        self.expect("frame var foo", "sanity check", substrs=['(Foo)'])
        self.expect("expr FromOverlay(i: 23)", error=True)

    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(oslist=['windows'])
    @swiftTest
    def test_extra_clang_flags(self):
        """
        Test that a debug-only module map can be installed by injecting a
        VFS overlay using target.swift-extra-clang-flags.
        """
        self.build()
        # FIXME: this doesn't work if LLDB's build dir contains a space.
        overlay = self.getBuildArtifact('overlay.yaml')
        self.addTearDownHook(
            lambda: self.runCmd("settings clear target.swift-extra-clang-flags"))
        self.expect('settings set -- target.swift-extra-clang-flags '+
                    '"-ivfsoverlay %s"' % overlay)
        with open(overlay, 'w+') as f:
            import os
            f.write("""
{
  'version': 0,
  'roots': [
    { 'name': '"""+os.getcwd()+"""/nonmodular', 'type': 'directory',
      'contents': [
        { 'name': 'module.modulemap', 'type': 'file',
          'external-contents': '"""+os.path.join(os.getcwd(),
                                                 'overlaid.map')+"""'
        }
      ]
    }
  ]
}
""")
        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        self.expect("frame var foo", "sanity check", substrs=['(Foo)'])
        self.expect("expr FromOverlay(i: 23)", substrs=['(FromOverlay)', '23'])

    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(oslist=['windows'])
    @swiftTest
    def test_invalid_extra_clang_flags(self):
        """
        Test that LLDB ignores specific invalid arguments in
        swift-extra-clang-flags.
        """
        self.build()
        self.addTearDownHook(
            lambda: self.runCmd("settings clear target.swift-extra-clang-flags"))

        self.expect('settings set target.swift-extra-clang-flags -- -v')

        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        self.expect("frame var foo", substrs=['(Foo)'])



# -*- coding: utf-8 -*-
"""
Test unicode handling in LLDB.
"""

import os

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestCase(PExpectTest):
    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])  # Randomly fails on buildbot
    def test_unicode_input(self):
        self.launch()

        # Send some unicode input to LLDB.
        # We should get back that this is an invalid command with our character as UTF-8.
        self.expect(
            "\u1234",
            substrs=["error: '\u1234' is not a valid command.".encode("utf-8")],
        )

        self.quit()



"""
Test the 'gui' default thread tree expansion.
The root process tree item and the tree item corresponding to the selected
thread should be expanded by default.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestGuiExpandThreadsTree(PExpectTest):
    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfCursesSupportMissing
    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])
    @skipIf(bugnumber="rdar://97460266")
    def test_gui(self):
        self.build()

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100, 500))
        self.expect(
            "breakpoint set -n break_here", substrs=["Breakpoint 1", "address ="]
        )
        self.expect("run", substrs=["stop reason ="])

        escape_key = chr(27).encode()

        # Start the GUI and close the welcome window.
        self.child.sendline("gui")
        self.child.send(escape_key)
        self.child.expect_exact("Threads")

        # The thread running thread_start_routine should be expanded.
        self.child.expect_exact("#0: break_here")

        # Exit GUI.
        self.child.send(escape_key)
        self.expect_prompt()

        # Select the main thread.
        self.child.sendline("thread select 1")

        # Start the GUI.
        self.child.sendline("gui")
        self.child.expect_exact("Threads")

        # The main thread should be expanded.
        self.child.expect("#\d+: main")

        # Quit the GUI
        self.child.send(escape_key)

        self.expect_prompt()
        self.quit()



from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            vec_type = "std::vector<int>"
        else:
            vec_type = "std::vector<int, std::allocator<int> >"

        # Test printing the vector before enabling any C++ module setting.
        self.expect_expr("a", result_type=vec_type)

        # Set loading the import-std-module to 'fallback' which loads the module
        # and retries when an expression fails to parse.
        self.runCmd("settings set target.import-std-module fallback")

        # Printing the vector still works. This should return the same type
        # as before as this shouldn't use a C++ module type.
        self.expect_expr("a", result_type=vec_type)

        # This expression can only parse with a C++ module. LLDB should
        # automatically fall back to import the C++ module to get this working.
        self.expect_expr("std::max<std::size_t>(0U, a.size())", result_value="3")

        # The 'a' and 'local' part can be parsed without loading a C++ module and will
        # load type/runtime information. The 'std::max...' part will fail to
        # parse without a C++ module. Make sure we reset all the relevant parts of
        # the C++ parser so that we don't end up with for example a second
        # definition of 'local' when retrying.
        self.expect_expr(
            "a; local; std::max<std::size_t>(0U, a.size())", result_value="3"
        )

        # Try to declare top-level declarations that require a C++ module to parse.
        # Top-level expressions don't support importing the C++ module (yet), so
        # this should still fail as before.
        self.expect(
            "expr --top-level -- int i = std::max(1, 2);",
            error=True,
            substrs=["no member named 'max' in namespace 'std'"],
        )

        # The proper diagnostic however should be shown on the retry.
        self.expect(
            "expr std::max(1, 2); unknown_identifier",
            error=True,
            substrs=["use of undeclared identifier 'unknown_identifier'"],
        )

        # Turn on the 'import-std-module' setting and make sure we import the
        # C++ module.
        self.runCmd("settings set target.import-std-module true")
        # This is still expected to work.
        self.expect_expr("std::max<std::size_t>(0U, a.size())", result_value="3")

        # Turn of the 'import-std-module' setting and make sure we don't load
        # the module (which should prevent parsing the expression involving
        # 'std::max').
        self.runCmd("settings set target.import-std-module false")
        self.expect(
            "expr std::max(1, 2);",
            error=True,
            substrs=["no member named 'max' in namespace 'std'"],
        )



"""Check that compiler-generated register values work correctly"""

import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


def re_expr_equals(val_type, val):
    # Match ({val_type}) ${sum_digits} = {val}
    return re.compile(r"\(" + val_type + "\) \$\d+ = " + str(val))


class RegisterVariableTestCase(TestBase):
    @expectedFailureAll(compiler="clang", compiler_version=["<", "3.5"])
    @expectedFailureAll(
        compiler="gcc", compiler_version=[">=", "4.8.2"], archs=["i386"]
    )
    @expectedFailureAll(compiler="gcc", compiler_version=["<", "4.9"], archs=["x86_64"])
    def test_and_run_command(self):
        """Test expressions on register values."""

        # This test now ensures that each probable
        # register variable location is actually a register, and
        # if so, whether we can print out the variable there.
        # It only requires one of them to be handled in a non-error
        # way.
        register_variables_count = 0

        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the main.
        lldbutil.run_break_set_by_source_regexp(self, "break", num_expected_locations=3)

        ####################
        # First breakpoint

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(
            self, bpno=1, location_id=1, expected_location_hit_count=1
        )

        # Try some variables that should be visible
        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        if self.is_variable_in_register(frame, "a"):
            register_variables_count += 1
            self.expect(
                "expr a",
                VARIABLES_DISPLAYED_CORRECTLY,
                patterns=[re_expr_equals("int", 2)],
            )

        if self.is_struct_pointer_in_register(frame, "b", self.TraceOn()):
            register_variables_count += 1
            self.expect(
                "expr b->m1",
                VARIABLES_DISPLAYED_CORRECTLY,
                patterns=[re_expr_equals("int", 3)],
            )

        #####################
        # Second breakpoint

        self.runCmd("continue")

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(
            self, bpno=1, location_id=2, expected_location_hit_count=1
        )

        # Try some variables that should be visible
        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        if self.is_struct_pointer_in_register(frame, "b", self.TraceOn()):
            register_variables_count += 1
            self.expect(
                "expr b->m2",
                VARIABLES_DISPLAYED_CORRECTLY,
                patterns=[re_expr_equals("int", 5)],
            )

        if self.is_variable_in_register(frame, "c"):
            register_variables_count += 1
            self.expect(
                "expr c",
                VARIABLES_DISPLAYED_CORRECTLY,
                patterns=[re_expr_equals("int", 5)],
            )

        #####################
        # Third breakpoint

        self.runCmd("continue")

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(
            self, bpno=1, location_id=3, expected_location_hit_count=1
        )

        # Try some variables that should be visible
        frame = (
            self.dbg.GetSelectedTarget()
            .GetProcess()
            .GetSelectedThread()
            .GetSelectedFrame()
        )
        if self.is_variable_in_register(frame, "f"):
            register_variables_count += 1
            self.expect(
                "expr f",
                VARIABLES_DISPLAYED_CORRECTLY,
                patterns=[re_expr_equals("float", "3.1")],
            )

        # Validate that we verified at least one register variable
        self.assertTrue(
            register_variables_count > 0,
            "expected to verify at least one variable in a register",
        )
        self.trace(
            "executed {} expressions with values in registers".format(
                register_variables_count
            )
        )

        self.runCmd("kill")

    def is_variable_in_register(self, frame, var_name):
        # Ensure we can lookup the variable.
        var = frame.FindVariable(var_name)
        self.trace("\nchecking {}...".format(var_name))
        if var is None or not var.IsValid():
            self.trace("{} cannot be found".format(var_name))
            return False

        # Check that we can get its value.  If not, this
        # may be a variable that is just out of scope at this point.
        value = var.GetValue()
        self.trace("checking value...")
        if value is None:
            self.trace("value is invalid")
            return False
        else:
            self.trace("value is {}".format(value))

        # We have a variable and we can get its value.  The variable is in a
        # register if we cannot get an address for it, assuming it is not a
        # struct pointer.  (This is an approximation - compilers can do other
        # things with spitting up a value into multiple parts of multiple
        # registers, but what we're verifying here is much more than it was
        # doing before).
        var_addr = var.GetAddress()
        self.trace("checking address...")
        if var_addr.IsValid():
            # We have an address, it must not be in a register.
            self.trace(
                "var {} is not in a register: has a valid address {}".format(
                    var_name, var_addr
                )
            )
            return False
        else:
            # We don't have an address but we can read the value.
            # It is likely stored in a register.
            self.trace(
                "var {} is in a register (we don't have an address for it)".format(
                    var_name
                )
            )
            return True

    def is_struct_pointer_in_register(self, frame, var_name, trace):
        # Ensure we can lookup the variable.
        var = frame.FindVariable(var_name)
        if trace:
            print("\nchecking {}...".format(var_name))

        if var is None or not var.IsValid():
            self.trace("{} cannot be found".format(var_name))
            return False

        # Check that we can get its value.  If not, this
        # may be a variable that is just out of scope at this point.
        value = var.GetValue()
        self.trace("checking value...")
        if value is None:
            if trace:
                print("value is invalid")
            return False
        else:
            if trace:
                print("value is {}".format(value))

        var_loc = var.GetLocation()
        if trace:
            print("checking location: {}".format(var_loc))
        if var_loc is None or var_loc.startswith("0x"):
            # The frame var is not in a register but rather a memory location.
            self.trace("frame var {} is not in a register".format(var_name))
            return False
        else:
            self.trace("frame var {} is in a register".format(var_name))
            return True



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects import pdl
from mlir.dialects.transform import loop


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        print(module)
    return f


@run
def getParentLoop():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.PROPAGATE, [], pdl.OperationType.get()
    )
    with InsertionPoint(sequence.body):
        loop.GetParentForOp(
            transform.OperationType.get("scf.for"), sequence.bodyTarget, num_loops=2
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: getParentLoop
    # CHECK: = transform.loop.get_parent_for %
    # CHECK: num_loops = 2


@run
def loopOutline():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.PROPAGATE,
        [],
        transform.OperationType.get("scf.for"),
    )
    with InsertionPoint(sequence.body):
        loop.LoopOutlineOp(
            transform.AnyOpType.get(),
            transform.AnyOpType.get(),
            sequence.bodyTarget,
            func_name="foo",
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: loopOutline
    # CHECK: = transform.loop.outline %
    # CHECK: func_name = "foo"


@run
def loopPeel():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.PROPAGATE,
        [],
        transform.OperationType.get("scf.for"),
    )
    with InsertionPoint(sequence.body):
        loop.LoopPeelOp(pdl.OperationType.get(), sequence.bodyTarget)
        transform.YieldOp()
    # CHECK-LABEL: TEST: loopPeel
    # CHECK: = transform.loop.peel %


@run
def loopPipeline():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.PROPAGATE,
        [],
        transform.OperationType.get("scf.for"),
    )
    with InsertionPoint(sequence.body):
        loop.LoopPipelineOp(
            pdl.OperationType.get(), sequence.bodyTarget, iteration_interval=3
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: loopPipeline
    # CHECK: = transform.loop.pipeline %
    # CHECK-DAG: iteration_interval = 3
    # (read_latency has default value and is not printed)


@run
def loopUnroll():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.PROPAGATE,
        [],
        transform.OperationType.get("scf.for"),
    )
    with InsertionPoint(sequence.body):
        loop.LoopUnrollOp(sequence.bodyTarget, factor=42)
        transform.YieldOp()
    # CHECK-LABEL: TEST: loopUnroll
    # CHECK: transform.loop.unroll %
    # CHECK: factor = 42



"""
Test some lldb command abbreviations to make sure the common short spellings of
many commands remain available even after we add/delete commands in the future.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CommonShortSpellingsTestCase(TestBase):
    @no_debug_info_test
    def test_abbrevs2(self):
        command_interpreter = self.dbg.GetCommandInterpreter()
        self.assertTrue(command_interpreter, VALID_COMMAND_INTERPRETER)
        result = lldb.SBCommandReturnObject()

        abbrevs = [
            ("br s", "breakpoint set"),
            ("disp", "_regexp-display"),  # a.k.a., 'display'
            ("di", "disassemble"),
            ("dis", "disassemble"),
            ("ta st a", "target stop-hook add"),
            ("fr v", "frame variable"),
            ("f 1", "frame select 1"),
            ("ta st li", "target stop-hook list"),
        ]

        for short_val, long_val in abbrevs:
            command_interpreter.ResolveCommand(short_val, result)
            self.assertTrue(result.Succeeded())
            self.assertEqual(long_val, result.GetOutput())



"""Test that inline functions from modules are imported correctly"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ModulesInlineFunctionsTestCase(TestBase):
    @add_test_categories(["gmodules"])
    @skipIf(macos_version=["<", "10.12"])
    def test_expr(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break inside the foo function which takes a bar_ptr argument.
        lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here.", lldb.SBFileSpec("main.m")
        )

        self.runCmd(
            'settings set target.clang-module-search-paths "'
            + self.getSourceDir()
            + '"'
        )

        self.expect(
            "expr @import myModule; 3",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["int", "3"],
        )

        self.expect("expr isInline(2)", VARIABLES_DISPLAYED_CORRECTLY, substrs=["4"])



# TestIndirectEnumVariables.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests that indirect Enum variables display correctly
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestIndirectEnumVariables(TestBase):
    @swiftTest
    def test_indirect_cases_variables(self):
        """Tests that indirect Enum variables display correctly when cases are indirect"""
        self.build()
        self.do_test("indirect case break here")

    @swiftTest
    def test_indirect_enum_variables(self):
        """Tests that indirect Enum variables display correctly when enum is indirect"""
        self.build()
        self.do_test("indirect enum break here")

    def setUp(self):
        TestBase.setUp(self)

    def get_variable(self, name):
        x = self.frame().FindVariable(name)
        x.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        x.SetPreferSyntheticValue(True)
        return x

    def check_enum(
            self,
            enum,
            value=None,
            summary=None,
            child_path=None,
            child_value=None,
            child_summary=None):
        if value:
            self.assertEqual(
                enum.GetValue(), value,
                "%s.GetValue() == %s" % (enum.GetName(), value),
            )
        if summary:
            self.assertEqual(
                enum.GetSummary(), summary,
                "%s.GetSummary() == %s" % (enum.GetName(), summary),
            )

        if child_path:
            child = enum
            for child_index in child_path:
                child = child.GetChildAtIndex(child_index)
                child.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
                child.SetPreferSyntheticValue(True)
            self.assertTrue(
                child.IsValid(),
                "child at path %s valid" %
                (child_path))
            if child_value:
                self.assertEqual(
                    child.GetValue(), child_value,
                    "%s.GetValue() == %s" % (child.GetName(), child_value),
                )
            if child_summary:
                self.assertEqual(
                    child.GetSummary(), child_summary,
                    "%s.GetSummary() == %s" % (child.GetName(), child_summary),
                )

    def do_test(self, break_pattern):
        """Tests that indirect Enum variables display correctly"""
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self,
            break_pattern,
            lldb.SBFileSpec("main.swift"),
        )

        GP_StructType = self.get_variable("GP_StructType")
        GP_TupleType = self.get_variable("GP_TupleType")
        GP_ClassType = self.get_variable("GP_ClassType")
        GP_ProtocolType_Struct = self.get_variable("GP_ProtocolType_Struct")
        GP_ProtocolType_Class = self.get_variable("GP_ProtocolType_Class")
        GP_CEnumType = self.get_variable("GP_CEnumType")
        GP_ADTEnumType = self.get_variable("GP_ADTEnumType")
        GP_Recursive = self.get_variable("GP_Recursive")

        self.check_enum(
            GP_StructType,
            value='StructType',
            child_path=[0],
            child_value='12')

        self.check_enum(
            GP_TupleType,
            value='TupleType',
            child_path=[
                0,
                0],
            child_value='12')
        self.check_enum(
            GP_TupleType, value='TupleType', child_path=[
                0, 1], child_summary='"Hello World"')

        self.check_enum(
            GP_ClassType, value='ClassType', child_path=[
                0, 0, 0], child_summary='"Hello World"')
        self.check_enum(
            GP_ClassType,
            value='ClassType',
            child_path=[
                0,
                1],
            child_value='12')

        self.check_enum(
            GP_ProtocolType_Struct,
            value='ProtocolType',
            child_path=[0],
            child_value='12')

        self.check_enum(
            GP_ProtocolType_Class,
            value='ProtocolType',
            child_path=[
                0,
                0,
                0],
            child_summary='"Hello World"')
        self.check_enum(
            GP_ProtocolType_Class,
            value='ProtocolType',
            child_path=[
                0,
                1],
            child_value='12')

        self.check_enum(
            GP_CEnumType,
            value='CEnumType',
            child_path=[0],
            child_value='B')

        self.check_enum(
            GP_ADTEnumType,
            value='ADTEnumType',
            child_path=[
                0,
                0],
            child_value='12')

        self.check_enum(
            GP_Recursive,
            value='Recursive',
            child_path=[
                0,
                0],
            child_value='12')




"""
Test that objective-c constant strings are generated correctly by the expression
parser.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ConstStringTestCase(TestBase):
    d = {"OBJC_SOURCES": "const-strings.m"}

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.main_source = "const-strings.m"
        self.line = line_number(self.main_source, "// Set breakpoint here.")

    def test_break(self):
        """Test constant string generation amd comparison by the expression parser."""
        self.build(dictionary=self.d)
        self.setTearDownCleanup(self.d)

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, self.main_source, self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)
        self.expect(
            "process status",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=[
                "stop reason = breakpoint",
                " at %s:%d" % (self.main_source, self.line),
            ],
        )

        self.expect('expression (int)[str compare:@"hello"]', startstr="(int) $0 = 0")
        self.expect('expression (int)[str compare:@"world"]', startstr="(int) $1 = -1")

        # Test empty strings, too.
        self.expect('expression (int)[@"" length]', startstr="(int) $2 = 0")

        self.expect('expression (int)[@"123" length]', startstr="(int) $3 = 3")



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import sys
import lldb


def fuzz_obj(obj):
    obj.GetID()
    obj.ClearAllBreakpointSites()
    obj.FindLocationByAddress(sys.maxsize)
    obj.FindLocationIDByAddress(sys.maxsize)
    obj.FindLocationByID(0)
    obj.GetLocationAtIndex(0)
    obj.SetEnabled(True)
    obj.IsEnabled()
    obj.GetHitCount()
    obj.SetIgnoreCount(1)
    obj.GetIgnoreCount()
    obj.SetCondition("i >= 10")
    obj.GetCondition()
    obj.SetThreadID(0)
    obj.GetThreadID()
    obj.SetThreadIndex(0)
    obj.GetThreadIndex()
    obj.SetThreadName("worker thread")
    obj.GetThreadName()
    obj.SetQueueName("my queue")
    obj.GetQueueName()
    obj.SetScriptCallbackFunction(None)
    obj.SetScriptCallbackBody(None)
    obj.GetNumResolvedLocations()
    obj.GetNumLocations()
    obj.GetDescription(lldb.SBStream())
    for bp_loc in obj:
        s = str(bp_loc)



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftAsyncVariables(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIf(oslist=['windows', 'linux'])
    def test(self):
        """Test local variables in async functions"""
        self.build()
        src = lldb.SBFileSpec('main.swift')
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', src)

        while process.selected_thread.stop_reason == lldb.eStopReasonBreakpoint:
            self.expect("frame variable x", substrs=["23"])
            process.Continue()



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()


# CHECK-LABEL: TEST: testNameIsPrivate
def testNameIsPrivate():
    # `import *` ignores private names starting with an understore, so the debug
    # flag shouldn't be visible unless explicitly imported.
    try:
        _GlobalDebug.flag = True
    except NameError:
        pass
    else:
        assert False, "_GlobalDebug must not be available by default"


run(testNameIsPrivate)


# CHECK-LABEL: TEST: testDebugDlag
def testDebugDlag():
    # Private names must be imported expilcitly.
    from mlir.ir import _GlobalDebug

    # CHECK: False
    print(_GlobalDebug.flag)
    _GlobalDebug.flag = True
    # CHECK: True
    print(_GlobalDebug.flag)
    _GlobalDebug.flag = False
    # CHECK: False
    print(_GlobalDebug.flag)


run(testDebugDlag)



# TestSwiftHideRuntimeSupport.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that we hide runtime support values
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftHideRuntimeSupport(TestBase):
    @swiftTest
    def test_swift_hide_runtime_support(self):
        """Test that we hide runtime support values"""

        # This is the function to remove the custom settings in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd(
                'settings set target.display-runtime-support-values true',
                check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("settings set target.display-runtime-support-values false")

        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        self.expect(
            'frame variable -d run',
            substrs=['_0_0'],
            matching=False)
        self.expect('frame variable -d run', substrs=['193627'], matching=True)

        var_opts = lldb.SBVariablesOptions()
        var_opts.SetIncludeArguments(True)
        var_opts.SetIncludeLocals(True)
        var_opts.SetInScopeOnly(True)
        var_opts.SetIncludeStatics(True)
        var_opts.SetIncludeRuntimeSupportValues(False)
        var_opts.SetUseDynamic(lldb.eDynamicCanRunTarget)

        values = self.frame().GetVariables(var_opts)
        found = False
        for value in values:
            if '_0_0' in value.name:
                found = True
            if '$' in value.name:
                found = True
        self.assertFalse(found, "found the thing I was not expecting")

        var_opts.SetIncludeRuntimeSupportValues(True)
        values = self.frame().GetVariables(var_opts)
        found = False
        for value in values:
            if '_0_0' in value.name:
                found = True
        self.assertTrue(found, "not found the thing I was expecting")

        self.runCmd("settings set target.display-runtime-support-values true")
        self.expect(
            'frame variable -d run',
            substrs=['_0_0'],
            matching=True)

        self.runCmd("settings set target.display-runtime-support-values false")
        self.expect(
            'frame variable -d run',
            substrs=['_0_0'],
            matching=False)



"""
Test source text regex breakpoint hydrates module debug info
in symbol on-demand mode.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestSourceTextRegexBreakpoint(TestBase):
    @skipIfWindows
    def test_with_run_command(self):
        self.build()

        # Load symbols on-demand
        self.runCmd("settings set symbols.load-on-demand true")

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_source_regexp(self, "Set break point at this line.")
        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        frame = self.frame()
        self.assertTrue(frame.IsValid())
        self.assertEqual(frame.GetLineEntry().GetFileSpec().GetFilename(), "main.cpp")
        self.assertEqual(frame.GetLineEntry().GetLine(), 4)



"""
Test that contiguous array prints correctly
"""
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestContiguousArray(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    def test_frame_contiguous_array(self):
        """Test that contiguous array prints correctly"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect("frame variable",
                    startstr="""(ContiguousArray<a.Class>) array = 1 value {
  [0] = 0x""")



"""
Tests large source files are not locked on Windows when source cache is disabled
"""

import lldb
import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from shutil import copy


class SettingsUseSourceCacheTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_set_use_source_cache_false(self):
        """Test that after 'set use-source-cache false', files are not locked."""
        self.set_use_source_cache_and_test(False)

    @skipIf(hostoslist=no_match(["windows"]))
    @skipIf(oslist=["windows"], archs=["aarch64"])  # Fails on windows 11
    def test_set_use_source_cache_true(self):
        """Test that after 'set use-source-cache false', files are locked."""
        self.set_use_source_cache_and_test(True)

    def set_use_source_cache_and_test(self, is_cache_enabled):
        """Common test for both True/False values of use-source-cache."""
        self.build()

        # Enable/Disable source cache
        self.runCmd(
            "settings set use-source-cache " + ("true" if is_cache_enabled else "false")
        )

        # Get paths for the main source file.
        src = self.getBuildArtifact("main-copy.cpp")
        self.assertTrue(src)

        # Make sure source file is bigger than 16K to trigger memory mapping
        self.assertGreater(os.stat(src).st_size, 4 * 4096)

        target, process, thread, breakpoint = lldbutil.run_to_name_breakpoint(
            self, "calc"
        )

        # Show the source file contents to make sure LLDB loads src file.
        self.runCmd("source list")

        # Try deleting the source file.
        is_file_removed = self.removeFile(src)

        if is_cache_enabled:
            self.assertFalse(
                is_file_removed, "Source cache is enabled, but delete file succeeded"
            )

        if not is_cache_enabled:
            self.assertTrue(
                is_file_removed, "Source cache is disabled, but delete file failed"
            )

    def removeFile(self, src):
        """Remove file and return true iff file was successfully removed."""
        try:
            os.remove(src)
            return True
        except Exception:
            return False



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftProtocolComposition(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        """Test that protocol composition types can be resolved
           through the Swift language runtime"""
        self.build()
        lldbutil.run_to_source_breakpoint(self, 'break here',
                                          lldb.SBFileSpec('main.swift'))
        obj = self.frame().FindVariable("obj", lldb.eDynamicDontRunTarget)
        p = obj.GetChildAtIndex(0, lldb.eDynamicDontRunTarget, False)
        some = p.GetChildAtIndex(0, lldb.eDynamicDontRunTarget, False)
        synth_obj = p.GetChildMemberWithName('object', lldb.eDynamicDontRunTarget)
        i = synth_obj.GetChildMemberWithName('i', lldb.eDynamicDontRunTarget)
        lldbutil.check_variable(self, i, value="23")



#!/usr/bin/env python

import lldb
import struct


class OperatingSystemPlugIn(object):
    """Class that provides a OS plugin that along with the particular code in main.cpp
    emulates the following scenario:
          a) We stop in an OS Plugin created thread - which should be thread index 1
          b) We step-out from that thread
          c) We hit a breakpoint in another thread, and DON'T produce the OS Plugin thread.
          d) We continue, and when we hit the step out breakpoint, we again produce the same
             OS Plugin thread.
          main.cpp sets values into the global variable g_value, which we use to tell the OS
          plugin whether to produce the OS plugin thread or not.
          Since we are always producing an OS plugin thread with a backing thread, we don't
          need to implement get_register_info or get_register_data.
    """

    def __init__(self, process):
        """Initialization needs a valid.SBProcess object.

        This plug-in will get created after a live process is valid and has stopped for the
        first time."""
        print("Plugin initialized.")
        self.process = None
        self.start_stop_id = 0
        self.g_value = lldb.SBValue()

        if isinstance(process, lldb.SBProcess) and process.IsValid():
            self.process = process
            self.g_value = process.GetTarget().FindFirstGlobalVariable("g_value")
            if not self.g_value.IsValid():
                print("Could not find g_value")

    def create_thread(self, tid, context):
        print("Called create thread with tid: ", tid)
        return None

    def get_thread_info(self):
        g_value = self.g_value.GetValueAsUnsigned()
        print("Called get_thread_info: g_value: %d" % (g_value))
        if g_value == 0 or g_value == 2:
            return [
                {
                    "tid": 0x111111111,
                    "name": "one",
                    "queue": "queue1",
                    "state": "stopped",
                    "stop_reason": "breakpoint",
                    "core": 1,
                }
            ]
        else:
            return []

    def get_register_info(self):
        print("called get_register_info")
        return None

    def get_register_data(self, tid):
        print("Get register data called for tid: %d" % (tid))
        return None



# TestSwiftFixIts.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test getting return values
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftFixIts(TestBase):
    @swiftTest
    def test_swift_fixits(self):
        """Test applying fixits to expressions"""
        self.build()
        self.do_test()

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def do_test(self):
        """Tests that we can break and display simple types"""
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here to test fixits', self.main_source_spec)
        self.assertTrue(breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        self.process = target.LaunchSimple(None, None, os.getcwd())

        self.assertTrue(self.process, PROCESS_IS_VALID)

        # Frame #0 should be at our breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, breakpoint)

        self.assertTrue(len(threads) == 1)
        self.thread = threads[0]

        frame = self.thread.frames[0]

        options = lldb.SBExpressionOptions()
        options.SetAutoApplyFixIts(False)

        # First make sure the expressions fail with no fixits:
        value = frame.EvaluateExpression(
            "var $tmp : Int = does_have.could_be!!", options)
        self.assertTrue(value.GetError().Fail())
        self.assertTrue(
            value.GetError().GetError() != 0x1001,
            'Failure was not "no return type"')

        value = frame.EvaluateExpression(
                "var $tmp2 = wrapper.wrapped", options)
        self.assertTrue(value.GetError().Fail())
        self.assertTrue(
            value.GetError().GetError() != 0x1001,
            'Failure was not "no return type"')

        # Now turn on auto apply:
        options.SetAutoApplyFixIts(True)
        value = frame.EvaluateExpression(
            "var $tmp : Int = does_have.could_be!!", options)
        self.assertTrue(value.GetError().Fail(),
                        "There's no result so this is counted a fail.")
        self.assertTrue(
            value.GetError().GetError() == 0x1001,
            'This error is "no return type"')

        # Check that the expressions were correct:
        tmp_value = frame.EvaluateExpression("$tmp == 100")
        self.assertSuccess(tmp_value.GetError())
        self.assertTrue(tmp_value.GetSummary() == 'true')
        
        value = frame.EvaluateExpression(
                "var $tmp2 = wrapper.wrapped", options)
        tmp_value = frame.EvaluateExpression("$tmp2 == 7")
        self.assertSuccess(tmp_value.GetError())
        self.assertTrue(tmp_value.GetSummary() == 'true')



"""Test the SBEnvironment APIs."""


from math import fabs
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SBEnvironmentAPICase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # We use this function to test both kind of accessors:
    # .  Get*AtIndex and GetEntries
    def assertEqualEntries(self, env, entries):
        self.assertEqual(env.GetNumValues(), len(entries))
        for i in range(env.GetNumValues()):
            name = env.GetNameAtIndex(i)
            value = env.GetValueAtIndex(i)
            self.assertIn(name + "=" + value, entries)

        entries = env.GetEntries()
        self.assertEqual(entries.GetSize(), len(entries))
        for i in range(entries.GetSize()):
            (name, value) = entries.GetStringAtIndex(i).split("=")
            self.assertIn(name + "=" + value, entries)

    @skipIfRemote  # Remote environment not supported.
    def test_platform_environment(self):
        env = self.dbg.GetSelectedPlatform().GetEnvironment()
        # We assume at least PATH is set
        self.assertNotEqual(env.Get("PATH"), None)

    def test_launch_info(self):
        target = self.dbg.CreateTarget("")
        launch_info = target.GetLaunchInfo()
        env = launch_info.GetEnvironment()
        env_count = env.GetNumValues()

        env.Set("FOO", "bar", True)
        self.assertEqual(env.GetNumValues(), env_count + 1)

        # Make sure we only modify the copy of the launchInfo's environment
        self.assertEqual(launch_info.GetEnvironment().GetNumValues(), env_count)

        # Note: swig >= 3.0.3 can use named parameters, omit to support <= 3.0.2.
        # i.e. launch_info.SetEnvironment(env, append=True)
        launch_info.SetEnvironment(env, True)
        self.assertEqual(launch_info.GetEnvironment().GetNumValues(), env_count + 1)

        env.Set("FOO", "baz", True)
        launch_info.SetEnvironment(env, True)
        self.assertEqual(launch_info.GetEnvironment().GetNumValues(), env_count + 1)
        self.assertEqual(launch_info.GetEnvironment().Get("FOO"), "baz")

        # Make sure we can replace the launchInfo's environment
        env.Clear()
        env.Set("BAR", "foo", True)
        env.PutEntry("X=y")
        launch_info.SetEnvironment(env, False)
        self.assertEqualEntries(launch_info.GetEnvironment(), ["BAR=foo", "X=y"])

    @skipIfRemote  # Remote environment not supported.
    def test_target_environment(self):
        env = self.dbg.GetSelectedTarget().GetEnvironment()
        # There is no target, so env should be empty
        self.assertEqual(env.GetNumValues(), 0)
        self.assertEqual(env.Get("PATH"), None)

        target = self.dbg.CreateTarget("")
        env = target.GetEnvironment()
        path = env.Get("PATH")
        # Now there's a target, so at least PATH should exist
        self.assertNotEqual(path, None)

        # Make sure we are getting a copy by modifying the env we just got
        env.PutEntry("PATH=#" + path)
        self.assertEqual(target.GetEnvironment().Get("PATH"), path)

    def test_creating_and_modifying_environment(self):
        env = lldb.SBEnvironment()

        self.assertEqual(env.Get("FOO"), None)
        self.assertEqual(env.Get("BAR"), None)

        # We also test empty values
        # Note: swig >= 3.0.3 can use named parameters, omit to support <= 3.0.2.
        # i.e. env.Set("FOO", "", overwrite=False)
        self.assertTrue(env.Set("FOO", "", False))
        env.Set("BAR", "foo", False)

        self.assertEqual(env.Get("FOO"), "")
        self.assertEqual(env.Get("BAR"), "foo")

        self.assertEqual(env.GetNumValues(), 2)

        self.assertEqualEntries(env, ["FOO=", "BAR=foo"])

        # Make sure modifications work
        self.assertFalse(env.Set("FOO", "bar", False))
        self.assertEqual(env.Get("FOO"), "")

        env.PutEntry("FOO=bar")
        self.assertEqual(env.Get("FOO"), "bar")

        self.assertEqualEntries(env, ["FOO=bar", "BAR=foo"])

        # Make sure we can unset
        self.assertTrue(env.Unset("FOO"))
        self.assertFalse(env.Unset("FOO"))
        self.assertEqual(env.Get("FOO"), None)

        # Test SetEntries
        entries = lldb.SBStringList()
        entries.AppendList(["X=x", "Y=y"], 2)

        env.SetEntries(entries, True)
        self.assertEqualEntries(env, ["BAR=foo", "X=x", "Y=y"])

        env.SetEntries(entries, False)
        self.assertEqualEntries(env, ["X=x", "Y=y"])

        entries.Clear()
        entries.AppendList(["X=y", "Y=x"], 2)
        env.SetEntries(entries, True)
        self.assertEqualEntries(env, ["X=y", "Y=x"])

        # Test clear
        env.Clear()
        self.assertEqualEntries(env, [])




"""
Test that a Swift class with C++ ivars is printed correctly
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropClassWithCxxIvars(TestBase):

    @swiftTest
    def test(self):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect('v swiftClass', substrs=['SwiftClass', 'cxxClass', 'a1', '10', 'a2', '20', 'a3', '30',
            'cxxSubclass', 'a1', '10', 'a2', '20', 'a3', '30', 'a4', '40'])
        self.expect('expr swiftClass', substrs=['SwiftClass', 'cxxClass', 'a1', '10', 'a2', '20', 'a3', '30',
            'cxxSubclass', 'a1', '10', 'a2', '20', 'a3', '30', 'a4', '40'])

        self.expect('v swiftStruct', substrs=['SwiftStruct', 'cxxClass', 'a1', '10', 'a2', '20', 'a3', '30',
            'cxxSubclass', 'a1', '10', 'a2', '20', 'a3', '30', 'a4', '40'])
        self.expect('expr swiftStruct', substrs=['SwiftStruct', 'cxxClass', 'a1', '10', 'a2', '20', 'a3', '30',
            'cxxSubclass', 'a1', '10', 'a2', '20', 'a3', '30', 'a4', '40'])


        self.expect('v swiftEnum1', substrs=['SwiftEnum', 'first', 'a1', '10', 'a2', '20', 'a3', '30'])
        self.expect('expr swiftEnum1', substrs=['SwiftEnum', 'first', 'a1', '10', 'a2', '20', 'a3', '30'])

        self.expect('v swiftEnum2', substrs=['SwiftEnum', 'second', 'a1', '10', 'a2', '20', 'a3', '30',
            'a4', '40'])
        self.expect('expr swiftEnum2', substrs=['SwiftEnum', 'second', 'a1', '10', 'a2', '20', 'a3', '30',
            'a4', '40'])



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftLetConstants(TestBase):
    @swiftTest
    def test_let_constants(self):
        """Test that let constants aren't writeable"""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        # Test globals.
        self.expect("expression  -- global_constant = 3333", error=True)
        self.expect("frame variable global_constant", substrs=['1111'])
        self.expect("expression  -- global_variable = 4444") # FIXME:, substrs=['4444'])
        self.expect("frame variable global_variable", substrs=['4444'])

        # Test function parameters.
        self.expect("expression  -- parameter_constant = 3333", error=True)
        self.expect("frame variable parameter_constant", substrs=['1111'])
        self.expect("expression  -- parameter_variable = 4444") # FIXME:, substrs=['4444'])
        self.expect("frame variable parameter_variable", substrs=['4444'])

        # Test local variables.
        self.expect("expression  -- local_constant = 3333", error=True)
        self.expect("frame variable local_constant", substrs=['1111'])
        self.expect("expression  -- local_variable = 4444") # FIXME:, substrs=['4444'])
        self.expect("frame variable local_variable", substrs=['4444'])



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftTripleDetection(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    def test(self):
        """Test that an underspecified triple is upgraded with a version number.
        """
        self.build()

        types_log = self.getBuildArtifact('types.log')
        self.runCmd('log enable lldb types -f "%s"' % types_log)
        exe = self.getBuildArtifact()
        arch = self.getArchitecture()
        target = self.dbg.CreateTargetWithFileAndTargetTriple(exe,
                                                              arch+"-apple-macos-unknown")
        bkpt = target.BreakpointCreateByName("main")
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.expect("expression 1")
        import io
        types_logfile = io.open(types_log, "r", encoding='utf-8')

        import re
        availability_re = re.compile(r'@available\(macCatalyst 1.*, \*\)')
        found = 0
        for line in types_logfile:
            print(line)
            if re.search('SwiftASTContextForExpressions.*Underspecified target triple .*-apple-macos-unknown', line):
                found += 1
                continue
            if found == 1 and re.search('SwiftASTContextForExpressions.*setting to ".*-apple-macos.[0-9.]+"', line):
               found += 1
               break
        self.assertEqual(found, 2)



"""
Test that the language option for breakpoints works correctly
with symbol load on-demand.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class TestBreakpointLanguage(TestBase):
    def check_location_file(self, bp, loc, test_name):
        bp_loc = bp.GetLocationAtIndex(loc)
        addr = bp_loc.GetAddress()
        comp_unit = addr.GetCompileUnit()
        comp_name = comp_unit.GetFileSpec().GetFilename()
        return comp_name == test_name

    @skipIfWindows
    def test_regex_breakpoint_language(self):
        """Test that the name regex breakpoint commands obey the language filter."""

        self.build()

        # Load symbols on-demand
        self.runCmd("settings set symbols.load-on-demand true")

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        error = lldb.SBError()
        # Don't read in dependencies so we don't come across false matches that
        # add unwanted breakpoint hits.
        self.target = self.dbg.CreateTarget(exe, None, None, False, error)
        self.assertTrue(self.target, VALID_TARGET)

        cpp_bp = self.target.BreakpointCreateByRegex(
            "func_from",
            lldb.eLanguageTypeC_plus_plus,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(cpp_bp.GetNumLocations(), 1, "Only one C++ symbol matches")
        self.assertTrue(self.check_location_file(cpp_bp, 0, "cpp_lang.cpp"))

        c_bp = self.target.BreakpointCreateByRegex(
            "func_from",
            lldb.eLanguageTypeC,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(c_bp.GetNumLocations(), 1, "Only one C symbol matches")
        self.assertTrue(self.check_location_file(c_bp, 0, "c_lang.c"))

        objc_bp = self.target.BreakpointCreateByRegex(
            "func_from",
            lldb.eLanguageTypeObjC,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(objc_bp.GetNumLocations(), 0, "No ObjC symbol matches")

    @skipIfWindows
    def test_by_name_breakpoint_language(self):
        """Test that the name regex breakpoint commands obey the language filter."""

        self.build()

        # Load symbols on-demand
        self.runCmd("settings set symbols.load-on-demand true")

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        error = lldb.SBError()
        # Don't read in dependencies so we don't come across false matches that
        # add unwanted breakpoint hits.
        self.target = self.dbg.CreateTarget(exe, None, None, False, error)
        self.assertTrue(self.target, VALID_TARGET)

        cpp_bp = self.target.BreakpointCreateByName(
            "func_from_cpp",
            lldb.eFunctionNameTypeAuto,
            lldb.eLanguageTypeC_plus_plus,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(cpp_bp.GetNumLocations(), 1, "Only one C++ symbol matches")
        self.assertTrue(self.check_location_file(cpp_bp, 0, "cpp_lang.cpp"))

        no_cpp_bp = self.target.BreakpointCreateByName(
            "func_from_c",
            lldb.eFunctionNameTypeAuto,
            lldb.eLanguageTypeC_plus_plus,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(no_cpp_bp.GetNumLocations(), 0, "And the C one doesn't match")

        c_bp = self.target.BreakpointCreateByName(
            "func_from_c",
            lldb.eFunctionNameTypeAuto,
            lldb.eLanguageTypeC,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(c_bp.GetNumLocations(), 1, "Only one C symbol matches")
        self.assertTrue(self.check_location_file(c_bp, 0, "c_lang.c"))

        no_c_bp = self.target.BreakpointCreateByName(
            "func_from_cpp",
            lldb.eFunctionNameTypeAuto,
            lldb.eLanguageTypeC,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(no_c_bp.GetNumLocations(), 0, "And the C++ one doesn't match")

        objc_bp = self.target.BreakpointCreateByName(
            "func_from_cpp",
            lldb.eFunctionNameTypeAuto,
            lldb.eLanguageTypeObjC,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(objc_bp.GetNumLocations(), 0, "No ObjC symbol matches")



"""
Test that using a non-existent architecture name does not crash LLDB.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class NoSuchArchTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Check that passing an invalid arch via the command-line fails but
        # doesn't crash
        self.expect(
            "target create --arch nothingtoseehere %s" % (exe),
            error=True,
            substrs=["error: invalid triple 'nothingtoseehere'"],
        )

        # Check that passing an invalid arch via the SB API fails but doesn't
        # crash
        target = self.dbg.CreateTargetWithFileAndArch(exe, "nothingtoseehere")
        self.assertFalse(target.IsValid(), "This target should not be valid")

        # Now just create the target with the default arch and check it's fine
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target.IsValid(), "This target should now be valid")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.arith as arith
import mlir.dialects.func as func
import mlir.dialects.tensor as tensor


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: testDimOp
@run
def testDimOp():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32Type = F32Type.get()
        indexType = IndexType.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get(
                    (ShapedType.get_dynamic_size(), ShapedType.get_dynamic_size()),
                    f32Type,
                )
            )
            #      CHECK: func @tensor_static_dim
            # CHECK-SAME:     %[[ARG0:.+]]: tensor<?x?xf32>
            #  CHECK-DAG:   %[[C0:.+]] = arith.constant 0 : index
            #  CHECK-DAG:   %[[C1:.+]] = arith.constant 1 : index
            #      CHECK:   %[[D0:.+]] = tensor.dim %[[ARG0]], %[[C0]]
            #      CHECK:   %[[D1:.+]] = tensor.dim %[[ARG0]], %[[C1]]
            #      CHECK:   return %[[D0]], %[[D1]]
            def tensor_static_dim(t):
                c0 = arith.ConstantOp(indexType, 0)
                c1 = arith.ConstantOp(indexType, 1)
                d0 = tensor.DimOp(t, c0)
                d1 = tensor.DimOp(t, c1)
                return [d0.result, d1.result]

        print(module)


# CHECK-LABEL: TEST: testEmptyOp
@run
def testEmptyOp():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):
            # CHECK-LABEL: func @static_sizes
            # CHECK: %0 = tensor.empty() : tensor<3x4xf32>
            @func.FuncOp.from_py_func()
            def static_sizes():
                return tensor.EmptyOp([3, 4], f32)

            # CHECK-LABEL: func @dynamic_sizes
            # CHECK: %0 = tensor.empty(%arg0, %arg1) : tensor<?x?xf32>
            @func.FuncOp.from_py_func(IndexType.get(), IndexType.get())
            def dynamic_sizes(d0, d1):
                return tensor.EmptyOp([d0, d1], f32)

            # CHECK-LABEL: func @mixed_static_dynamic_sizes
            # CHECK: %0 = tensor.empty(%arg0) : tensor<?x4xf32>
            @func.FuncOp.from_py_func(IndexType.get())
            def mixed_static_dynamic_sizes(d0):
                return tensor.EmptyOp([d0, 4], f32)

            # CHECK-LABEL: func @zero_d
            # CHECK: %0 = tensor.empty() : tensor<f32>
            @func.FuncOp.from_py_func()
            def zero_d():
                return tensor.EmptyOp([], f32)

    print(module)


# CHECK-LABEL: TEST: testInferTypesInsertSlice
@run
def testInferTypesInsertSlice():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32Type = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func(
                RankedTensorType.get((1, 1), f32Type),
                RankedTensorType.get((1, 1), f32Type),
            )
            # CHECK: func @f
            # CHECK:      tensor.insert_slice %arg0 into %arg1[0, 0] [1, 1] [0, 0] :
            # CHECK-SAME:   tensor<1x1xf32> into tensor<1x1xf32>
            def f(source, dest):
                d0 = tensor.InsertSliceOp(
                    source,
                    dest,
                    [],
                    [],
                    [],
                    DenseI64ArrayAttr.get([0, 0]),
                    DenseI64ArrayAttr.get([1, 1]),
                    DenseI64ArrayAttr.get([0, 0]),
                )
                return [d0.result]

    print(module)


# CHECK-LABEL: TEST: testFromElementsOp
@run
def testFromElementsOp():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):

            @func.FuncOp.from_py_func()
            def default_builder():
                c0 = arith.ConstantOp(f32, 0.0)
                # CHECK: %[[C0:.*]] = "arith.constant
                # CHECK-SAME: value = 0.000000e+00 : f32
                print(c0)
                c1 = arith.ConstantOp(f32, 1.0)
                # CHECK: %[[C1:.*]] = "arith.constant
                # CHECK-SAME: value = 1.000000e+00 : f32
                print(c1)

                t = tensor.FromElementsOp(RankedTensorType.get((2,), f32), [c0, c1])
                # CHECK: %{{.*}} = "tensor.from_elements"(%[[C0]], %[[C1]]) : (f32, f32) -> tensor<2xf32>
                print(t)

                t = tensor.FromElementsOp(RankedTensorType.get((2, 1), f32), [c0, c1])
                # CHECK: %{{.*}} = "tensor.from_elements"(%[[C0]], %[[C1]]) : (f32, f32) -> tensor<2x1xf32>
                print(t)

                t = tensor.FromElementsOp(RankedTensorType.get((1, 2), f32), [c0, c1])
                # CHECK: %{{.*}} = "tensor.from_elements"(%[[C0]], %[[C1]]) : (f32, f32) -> tensor<1x2xf32>
                print(t)



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbbench import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBenchmarkLibcxxList(BenchBase):
    @benchmarks_test
    def test_run_command(self):
        """Benchmark the std::list data formatter (libc++)"""
        self.build()
        self.data_formatter_commands()

    def setUp(self):
        # Call super's setUp().
        BenchBase.setUp(self)

    def data_formatter_commands(self):
        """Benchmark the std::list data formatter (libc++)"""
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        bkpt = self.target().FindBreakpointByID(
            lldbutil.run_break_set_by_source_regexp(self, "break here")
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This is the function to remove the custom formats in order to have a
        # clean slate for the next test case.
        def cleanup():
            self.runCmd("type format clear", check=False)
            self.runCmd("type summary clear", check=False)
            self.runCmd("type filter clear", check=False)
            self.runCmd("type synth clear", check=False)
            self.runCmd("settings set target.max-children-count 256", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        sw = Stopwatch()

        sw.start()
        self.expect("frame variable -A list", substrs=["[300]", "300"])
        sw.stop()

        print("time to print: %s" % (sw))



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class MyResponder(MockGDBServerResponder):
    """
    A responder which simulates a process with a single shared library loaded.
    Its parameters allow configuration of various properties of the library.
    """

    def __init__(self, testcase, triple, library_name, auxv_entry, region_info):
        MockGDBServerResponder.__init__(self)
        self.testcase = testcase
        self._triple = triple
        self._library_name = library_name
        self._auxv_entry = auxv_entry
        self._region_info = region_info

    def qSupported(self, client_supported):
        return (
            super().qSupported(client_supported)
            + ";qXfer:auxv:read+;qXfer:libraries-svr4:read+"
        )

    def qXferRead(self, obj, annex, offset, length):
        if obj == "features" and annex == "target.xml":
            return (
                """<?xml version="1.0"?>
                <target version="1.0">
                  <architecture>i386:x86-64</architecture>
                  <feature name="org.gnu.gdb.i386.core">
                    <reg name="rip" bitsize="64" regnum="0" type="code_ptr" group="general"/>
                  </feature>
                </target>""",
                False,
            )
        elif obj == "auxv":
            # 0x09 = AT_ENTRY, which lldb uses to compute the load bias of the
            # main binary.
            return (
                hex_decode_bytes(
                    self._auxv_entry
                    + "09000000000000000000ee000000000000000000000000000000000000000000"
                ),
                False,
            )
        elif obj == "libraries-svr4":
            return (
                """<?xml version="1.0"?>
                <library-list-svr4 version="1.0">
                  <library name="%s" lm="0xdeadbeef" l_addr="0xef0000" l_ld="0xdeadbeef"/>
                </library-list-svr4>"""
                % self._library_name,
                False,
            )
        else:
            return None, False

    def qfThreadInfo(self):
        return "m47"

    def qsThreadInfo(self):
        return "l"

    def qProcessInfo(self):
        return "pid:47;ptrsize:8;endian:little;triple:%s;" % hex_encode_bytes(
            self._triple
        )

    def setBreakpoint(self, packet):
        return "OK"

    def readMemory(self, addr, length):
        if addr == 0xEE1000:
            return "00" * 0x30 + "0020ee0000000000"
        elif addr == 0xEE2000:
            return "01000000000000000030ee0000000000dead00000000000000000000000000000000000000000000"
        elif addr == 0xEF0000:
            with open(self.testcase.getBuildArtifact("libmodule_load.so"), "rb") as f:
                contents = f.read(-1)
            return hex_encode_bytes(seven.bitcast_to_string(contents))
        return ("baadf00d00" * 1000)[0 : length * 2]

    def qMemoryRegionInfo(self, addr):
        if addr < 0xEE0000:
            return "start:0;size:ee0000;"
        elif addr < 0xEF0000:
            return "start:ee0000;size:10000;"
        elif addr < 0xF00000:
            return "start:ef0000;size:1000;permissions:rx;" + self._region_info
        else:
            return "start:ef1000;size:ffffffffff10f000"


class TestGdbClientModuleLoad(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    def test_android_app_process(self):
        """
        This test simulates the scenario where the (android) dynamic linker
        reports incorrect file name of the main executable. Lldb uses
        qMemoryRegionInfo to get the correct value.
        """
        region_info = "name:%s;" % (
            hex_encode_bytes(self.getBuildArtifact("libmodule_load.so"))
        )
        self.server.responder = MyResponder(
            self, "x86_64-pc-linux-android", "bogus-name", "", region_info
        )
        self.yaml2obj("module_load.yaml", self.getBuildArtifact("libmodule_load.so"))
        target = self.createTarget("module_load.yaml")

        process = self.connect(target)
        self.assertTrue(process.IsValid(), "Process is valid")

        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        self.filecheck("image list", __file__, "-check-prefix=ANDROID")

    # ANDROID: [  0] {{.*}} 0x0000000000ee0000 {{.*}}module_load
    # ANDROID: [  1] {{.*}} 0x0000000000ef0000 {{.*}}libmodule_load.so

    @skipIfXmlSupportMissing
    def test_vdso(self):
        """
        This test checks vdso loading in the situation where the process does
        not have memory region information about the vdso address. This can
        happen in core files, as they don't store this data.
        We want to check that the vdso is loaded exactly once.
        """
        # vdso address
        AT_SYSINFO_EHDR = "21000000000000000000ef0000000000"
        self.server.responder = MyResponder(
            self, "x86_64-pc-linux", "linux-vdso.so.1", AT_SYSINFO_EHDR, ""
        )
        self.yaml2obj("module_load.yaml", self.getBuildArtifact("libmodule_load.so"))
        target = self.createTarget("module_load.yaml")

        process = self.connect(target)
        self.assertTrue(process.IsValid(), "Process is valid")

        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        self.filecheck("image list", __file__, "-check-prefix=VDSO")
        # VDSO: [  0] {{.*}} 0x0000000000ee0000 {{.*}}module_load
        # VDSO: [  1] {{.*}} 0x0000000000ef0000 {{.*}}[vdso]
        self.assertEqual(self.target().GetNumModules(), 2)



"""
Tests that LLDB uses the external type info provider for C types.
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2



class TestSwiftCTypeExternalProvider(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    @swiftTest
    def test_swift_regex(self):
        """Test that C types with builtin metadata emitted are looked up using
        external type info provider."""
        self.build()
        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', self.main_source_spec)

        # Consult the second field to ensure we call GetIndexOfChildMemberWithName.
        self.expect('v dummy.second', substrs=['2'])

        # Make sure we look up the type with the external type info provider.
        provider_log_found = False
        with open(log, "r", encoding='utf-8') as logfile:
            for line in logfile:
                if '[LLDBTypeInfoProvider] Looking up debug type info for So5DummyV' in line:
                    provider_log_found = True
                    break
        self.assertTrue(provider_log_found)





"""
Test that evaluating expressions works on forward interop mode.
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropExpressions(TestBase):

    def setup(self, bkpt_str):
         self.build()
         
         _, _, thread, _ = lldbutil.run_to_source_breakpoint(
             self, bkpt_str, lldb.SBFileSpec('main.swift'))
         return thread

    @skipIfLinux # rdar://106871422"
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false')) # rdar://106871275
    @swiftTest
    def test(self):
        self.setup('Break here')
        
        # Check that we can call free functions.
        self.expect('expr returnsInt()', substrs=['Int32', '42'])

        # Check that we can call unused free functions.
        self.expect('expr returnsIntUnused()', substrs=['Int32', '37'])

        # Check that we can call a C++ constructor.
        self.expect('expr CxxClass()', substrs=['CxxClass', 'a = 100', 'b = 101'])

        # Check that we can call methods.
        self.expect('expr cxxClass.sum()', substrs=['Int32', '201'])

        # Check that we can access a C++ type's ivars
        self.expect('expr cxxClass.a', substrs=['Int32', '100'])
        self.expect('expr cxxSubclass.a', substrs=['Int32', '100'])
        self.expect('expr cxxSubclass.c', substrs=['Int32', '102'])

        # Check that calling a function that throws an exception fails on expression evaluation
        self.expect('expr throwException()', substrs=['internal c++ exception breakpoint'], error=True)

        # Check that we can make persistent variables.
        self.expect('expr var $cxxClass = CxxClass()')

        # Check that we can refer to the persistent variable.
        self.expect('expr $cxxClass', substrs=['CxxClass', 'a = 100', 'b = 101'])

        # Check that we can call methods on the persistent variable.
        self.expect('expr $cxxClass.sum()', substrs=['Int32', '201'])

        # Check that po prints the fields of a base class
        self.expect('po cxxClass', substrs=['CxxClass', 'a : 100', 'b : 101'])

    @expectedFailureAll(bugnumber="rdar://106216567")
    @swiftTest
    def test_po_subclass(self):
        self.setup('Break here')

        self.expect('po CxxSubclass()', substrs=['CxxClass', 'a : 100', 'b : 101', 'c : 102'])




##===-- commandwin.py ----------------------------------------*- Python -*-===##
##
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
##
##===----------------------------------------------------------------------===##

import cui
import curses
import lldb
from itertools import islice


class History(object):

    def __init__(self):
        self.data = {}
        self.pos = 0
        self.tempEntry = ''

    def previous(self, curr):
        if self.pos == len(self.data):
            self.tempEntry = curr

        if self.pos < 0:
            return ''
        if self.pos == 0:
            self.pos -= 1
            return ''
        if self.pos > 0:
            self.pos -= 1
            return self.data[self.pos]

    def next(self):
        if self.pos < len(self.data):
            self.pos += 1

        if self.pos < len(self.data):
            return self.data[self.pos]
        elif self.tempEntry != '':
            return self.tempEntry
        else:
            return ''

    def add(self, c):
        self.tempEntry = ''
        self.pos = len(self.data)
        if self.pos == 0 or self.data[self.pos - 1] != c:
            self.data[self.pos] = c
            self.pos += 1


class CommandWin(cui.TitledWin):

    def __init__(self, driver, x, y, w, h):
        super(CommandWin, self).__init__(x, y, w, h, "Commands")
        self.command = ""
        self.data = ""
        driver.setSize(w, h)

        self.win.scrollok(1)

        self.driver = driver
        self.history = History()

        def enterCallback(content):
            self.handleCommand(content)

        def tabCompleteCallback(content):
            self.data = content
            matches = lldb.SBStringList()
            commandinterpreter = self.getCommandInterpreter()
            commandinterpreter.HandleCompletion(
                self.data, self.el.index, 0, -1, matches)
            if matches.GetSize() == 2:
                self.el.content += matches.GetStringAtIndex(0)
                self.el.index = len(self.el.content)
                self.el.draw()
            else:
                self.win.move(self.el.starty, self.el.startx)
                self.win.scroll(1)
                self.win.addstr("Available Completions:")
                self.win.scroll(1)
                for m in islice(matches, 1, None):
                    self.win.addstr(self.win.getyx()[0], 0, m)
                    self.win.scroll(1)
                self.el.draw()

        self.startline = self.win.getmaxyx()[0] - 2

        self.el = cui.CursesEditLine(
            self.win,
            self.history,
            enterCallback,
            tabCompleteCallback)
        self.el.prompt = self.driver.getPrompt()
        self.el.showPrompt(self.startline, 0)

    def handleCommand(self, cmd):
       # enter!
        self.win.scroll(1)  # TODO: scroll more for longer commands
        if cmd == '':
            cmd = self.history.previous('')
        elif cmd in ('q', 'quit'):
            self.driver.terminate()
            return

        self.history.add(cmd)
        ret = self.driver.handleCommand(cmd)
        if ret.Succeeded():
            out = ret.GetOutput()
            attr = curses.A_NORMAL
        else:
            out = ret.GetError()
            attr = curses.color_pair(3)  # red on black
        self.win.addstr(self.startline, 0, out + '\n', attr)
        self.win.scroll(1)
        self.el.showPrompt(self.startline, 0)

    def handleEvent(self, event):
        if isinstance(event, int):
            if event == curses.ascii.EOT and self.el.content == '':
                # When the command is empty, treat CTRL-D as EOF.
                self.driver.terminate()
                return
            self.el.handleEvent(event)

    def getCommandInterpreter(self):
        return self.driver.getCommandInterpreter()



# RUN: env SUPPORTLIB=%mlir_c_runner_utils %PYTHON %s | FileCheck %s

import numpy as np
import os
import sys
import tempfile

_SCRIPT_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(_SCRIPT_PATH)

from tools import mlir_pytaco_api as pt
from tools import testing_utils as utils

###### This PyTACO part is taken from the TACO open-source project. ######
# See http://tensor-compiler.org/docs/scientific_computing/index.html.

compressed = pt.compressed
dense = pt.dense

# Define formats for storing the sparse matrix and dense vectors.
csr = pt.format([dense, compressed])
dv = pt.format([dense])

# Load a sparse matrix stored in the matrix market format) and store it
# as a CSR matrix.  The matrix in this test is a reduced version of the data
# downloaded from here:
# https://www.cise.ufl.edu/research/sparse/MM/Boeing/pwtk.tar.gz
# In order to run the program using the matrix above, you can download the
# matrix and replace this path to the actual path to the file.
A = pt.read(os.path.join(_SCRIPT_PATH, "data/pwtk.mtx"), csr)

# These two lines have been modified from the original program to use static
# data to support result comparison.
x = pt.from_array(np.full((A.shape[1],), 1, dtype=np.float32))
z = pt.from_array(np.full((A.shape[0],), 2, dtype=np.float32))

# Declare the result to be a dense vector
y = pt.tensor([A.shape[0]], dv)

# Declare index vars
i, j = pt.get_index_vars(2)

# Define the SpMV computation
y[i] = A[i, j] * x[j] + z[i]

##########################################################################

# Perform the SpMV computation and write the result to file
with tempfile.TemporaryDirectory() as test_dir:
    golden_file = os.path.join(_SCRIPT_PATH, "data/gold_y.tns")
    out_file = os.path.join(test_dir, "y.tns")
    pt.write(out_file, y)
    #
    # CHECK: Compare result True
    #
    print(f"Compare result {utils.compare_sparse_tns(golden_file, out_file)}")



from lldbsuite.test import lldbinline
from lldbsuite.test import decorators

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    [
        decorators.expectedFailureAll(bugnumber="llvm.org/pr50814", compiler="gcc"),
    ],
)



"""
Test diamond inheritance.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_with_sbvalue(self):
        """
        Test that virtual base classes work in when SBValue objects are
        used to explore the class.
        """
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// breakpoint 1", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.prefer-dynamic-value no-dynamic-values")

        j1 = self.frame().FindVariable("j1")
        j1_Derived1 = j1.GetChildAtIndex(0)
        j1_Derived2 = j1.GetChildAtIndex(1)
        j1_Derived1_VBase = j1_Derived1.GetChildAtIndex(0)
        j1_Derived2_VBase = j1_Derived2.GetChildAtIndex(0)
        j1_Derived1_VBase_m_value = j1_Derived1_VBase.GetChildAtIndex(0)
        j1_Derived2_VBase_m_value = j1_Derived2_VBase.GetChildAtIndex(0)

        self.assertEqual(
            j1_Derived1_VBase.GetLoadAddress(),
            j1_Derived2_VBase.GetLoadAddress(),
            "ensure virtual base class is the same between Derived1 and Derived2",
        )
        self.assertEqual(
            j1_Derived1_VBase_m_value.GetValueAsUnsigned(1),
            j1_Derived2_VBase_m_value.GetValueAsUnsigned(2),
            "ensure m_value in VBase is the same",
        )
        self.assertEqual(
            self.frame()
            .FindVariable("d")
            .GetChildAtIndex(0)
            .GetChildAtIndex(0)
            .GetValueAsUnsigned(0),
            12345,
            "ensure Derived2 from j1 is correct",
        )

        # This reassigns 'd' to point to 'j2'.
        self.thread().StepOver()

        self.assertEqual(
            self.frame()
            .FindVariable("d")
            .GetChildAtIndex(0)
            .GetChildAtIndex(0)
            .GetValueAsUnsigned(0),
            12346,
            "ensure Derived2 from j2 is correct",
        )

    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// breakpoint 1", lldb.SBFileSpec("main.cpp")
        )

        # All the children of j1.
        children = [
            ValueCheck(
                type="Derived1",
                children=[
                    ValueCheck(
                        type="VBase",
                        children=[
                            ValueCheck(type="int", name="m_value", value="12345")
                        ],
                    )
                ],
            ),
            ValueCheck(
                type="Derived2",
                children=[
                    ValueCheck(
                        type="VBase",
                        children=[
                            ValueCheck(type="int", name="m_value", value="12345")
                        ],
                    )
                ],
            ),
            ValueCheck(type="long", value="1"),
        ]
        # Try using the class with expression evaluator/variable paths.
        self.expect_expr("j1", result_type="Joiner1", result_children=children)
        self.expect_var_path("j1", type="Joiner1", children=children)

        # Use the expression evaluator to access the members.
        self.expect_expr("j1.x", result_type="long", result_value="1")
        self.expect_expr("j1.m_value", result_type="int", result_value="12345")

        # Use variable paths to access the members.
        self.expect_var_path("j1.x", type="long", value="1")

    @expectedFailureAll
    @no_debug_info_test
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// breakpoint 1", lldb.SBFileSpec("main.cpp")
        )
        # FIXME: This is completely broken and 'succeeds' with an error that
        # there is noch such value/member in Joiner1. Move this up to the test
        # above when fixed.
        self.expect_var_path("j1.m_value", type="int", value="12345")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import arith
from mlir.dialects import builtin
from mlir.dialects import func


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()
        print(module)
    return f


# CHECK-LABEL: TEST: testConstantOp


@constructAndPrintInModule
def testConstantOp():
    c1 = arith.ConstantOp(IntegerType.get_signless(32), 42)
    c2 = arith.ConstantOp(IntegerType.get_signless(64), 100)
    c3 = arith.ConstantOp(F32Type.get(), 3.14)
    c4 = arith.ConstantOp(F64Type.get(), 1.23)
    # CHECK: 42
    print(c1.literal_value)

    # CHECK: 100
    print(c2.literal_value)

    # CHECK: 3.140000104904175
    print(c3.literal_value)

    # CHECK: 1.23
    print(c4.literal_value)


# CHECK: = arith.constant 42 : i32
# CHECK: = arith.constant 100 : i64
# CHECK: = arith.constant 3.140000e+00 : f32
# CHECK: = arith.constant 1.230000e+00 : f64


# CHECK-LABEL: TEST: testVectorConstantOp
@constructAndPrintInModule
def testVectorConstantOp():
    int_type = IntegerType.get_signless(32)
    vec_type = VectorType.get([2, 2], int_type)
    c1 = arith.ConstantOp(
        vec_type, DenseElementsAttr.get_splat(vec_type, IntegerAttr.get(int_type, 42))
    )
    try:
        print(c1.literal_value)
    except ValueError as e:
        assert "only integer and float constants have literal values" in str(e)
    else:
        assert False


# CHECK: = arith.constant dense<42> : vector<2x2xi32>


# CHECK-LABEL: TEST: testConstantIndexOp
@constructAndPrintInModule
def testConstantIndexOp():
    c1 = arith.ConstantOp.create_index(10)
    # CHECK: 10
    print(c1.literal_value)


# CHECK: = arith.constant 10 : index


# CHECK-LABEL: TEST: testFunctionCalls
@constructAndPrintInModule
def testFunctionCalls():
    foo = func.FuncOp("foo", ([], []))
    foo.sym_visibility = StringAttr.get("private")
    bar = func.FuncOp("bar", ([], [IndexType.get()]))
    bar.sym_visibility = StringAttr.get("private")
    qux = func.FuncOp("qux", ([], [F32Type.get()]))
    qux.sym_visibility = StringAttr.get("private")

    with InsertionPoint(func.FuncOp("caller", ([], [])).add_entry_block()):
        func.CallOp(foo, [])
        func.CallOp([IndexType.get()], "bar", [])
        func.CallOp([F32Type.get()], FlatSymbolRefAttr.get("qux"), [])
        func.ReturnOp([])


# CHECK: func private @foo()
# CHECK: func private @bar() -> index
# CHECK: func private @qux() -> f32
# CHECK: func @caller() {
# CHECK:   call @foo() : () -> ()
# CHECK:   %0 = call @bar() : () -> index
# CHECK:   %1 = call @qux() : () -> f32
# CHECK:   return
# CHECK: }



"""
Test lldb process launch flags.
"""

import os

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ProcessLaunchTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.runCmd("settings set auto-confirm true")

    def tearDown(self):
        self.runCmd("settings clear auto-confirm")
        TestBase.tearDown(self)

    @skipIfRemote
    def test_io(self):
        """Test that process launch I/O redirection flags work properly."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        in_file = os.path.join(self.getSourceDir(), "input-file.txt")
        out_file = lldbutil.append_to_process_working_directory(self, "output-test.out")
        err_file = lldbutil.append_to_process_working_directory(self, "output-test.err")

        # Make sure the output files do not exist before launching the process
        try:
            os.remove(out_file)
        except OSError:
            pass

        try:
            os.remove(err_file)
        except OSError:
            pass

        launch_command = "process launch -i '{0}' -o '{1}' -e '{2}' -w '{3}'".format(
            in_file, out_file, err_file, self.get_process_working_directory()
        )

        if lldb.remote_platform:
            self.runCmd(
                'platform put-file "{local}" "{remote}"'.format(
                    local=in_file, remote=in_file
                )
            )

        self.expect(launch_command, patterns=["Process .* launched: .*a.out"])

        success = True
        err_msg = ""

        out = lldbutil.read_file_on_target(self, out_file)
        if out != "This should go to stdout.\n":
            success = False
            err_msg = (
                err_msg + "    ERROR: stdout file does not contain correct output.\n"
            )

        err = lldbutil.read_file_on_target(self, err_file)
        if err != "This should go to stderr.\n":
            success = False
            err_msg = (
                err_msg + "    ERROR: stderr file does not contain correct output.\n"
            )

        if not success:
            self.fail(err_msg)

    # rdar://problem/9056462
    # The process launch flag '-w' for setting the current working directory
    # not working?
    @skipIfRemote
    @expectedFailureAll(oslist=["freebsd", "linux"], bugnumber="llvm.org/pr20265")
    @expectedFailureNetBSD
    def test_set_working_dir_nonexisting(self):
        """Test that '-w dir' fails to set the working dir when running the inferior with a dir which doesn't exist."""
        d = {"CXX_SOURCES": "print_cwd.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(d)
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe)

        mywd = "my_working_dir"
        out_file_name = "my_working_dir_test.out"
        err_file_name = "my_working_dir_test.err"

        my_working_dir_path = self.getBuildArtifact(mywd)
        out_file_path = os.path.join(my_working_dir_path, out_file_name)
        err_file_path = os.path.join(my_working_dir_path, err_file_name)

        # Check that we get an error when we have a nonexisting path
        invalid_dir_path = mywd + "z"
        launch_command = "process launch -w %s -o %s -e %s" % (
            invalid_dir_path,
            out_file_path,
            err_file_path,
        )

        self.expect(
            launch_command,
            error=True,
            patterns=["error:.* No such file or directory: %s" % invalid_dir_path],
        )

    @skipIfRemote
    def test_set_working_dir_existing(self):
        """Test that '-w dir' sets the working dir when running the inferior."""
        d = {"CXX_SOURCES": "print_cwd.cpp"}
        self.build(dictionary=d)
        self.setTearDownCleanup(d)
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe)

        mywd = "my_working_dir"
        out_file_name = "my_working_dir_test.out"
        err_file_name = "my_working_dir_test.err"

        my_working_dir_path = self.getBuildArtifact(mywd)
        lldbutil.mkdir_p(my_working_dir_path)
        out_file_path = os.path.join(my_working_dir_path, out_file_name)
        err_file_path = os.path.join(my_working_dir_path, err_file_name)

        # Make sure the output files do not exist before launching the process
        try:
            os.remove(out_file_path)
            os.remove(err_file_path)
        except OSError:
            pass

        launch_command = "process launch -w %s -o %s -e %s" % (
            my_working_dir_path,
            out_file_path,
            err_file_path,
        )

        self.expect(launch_command, patterns=["Process .* launched: .*a.out"])

        success = True
        err_msg = ""

        # Check to see if the 'stdout' file was created
        try:
            out_f = open(out_file_path)
        except IOError:
            success = False
            err_msg = err_msg + "ERROR: stdout file was not created.\n"
        else:
            # Check to see if the 'stdout' file contains the right output
            line = out_f.readline()
            if self.TraceOn():
                print("line:", line)
            if not re.search(mywd, line):
                success = False
                err_msg = (
                    err_msg + "The current working directory was not set correctly.\n"
                )
                out_f.close()

        # Try to delete the 'stdout' and 'stderr' files
        try:
            os.remove(out_file_path)
            os.remove(err_file_path)
        except OSError:
            pass

        if not success:
            self.fail(err_msg)

    def test_environment_with_special_char(self):
        """Test that environment variables containing '*' and '}' are handled correctly by the inferior."""
        source = "print_env.cpp"
        d = {"CXX_SOURCES": source}
        self.build(dictionary=d)
        self.setTearDownCleanup(d)

        evil_var = "INIT*MIDDLE}TAIL"

        target = self.createTestTarget()
        main_source_spec = lldb.SBFileSpec(source)
        breakpoint = target.BreakpointCreateBySourceRegex(
            "// Set breakpoint here.", main_source_spec
        )

        process = target.LaunchSimple(
            None, ["EVIL=" + evil_var], self.get_process_working_directory()
        )
        self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint)
        self.assertEqual(len(threads), 1)
        frame = threads[0].GetFrameAtIndex(0)
        sbvalue = frame.EvaluateExpression("evil")
        value = sbvalue.GetSummary().strip('"')

        self.assertEqual(value, evil_var)
        process.Continue()
        self.assertState(process.GetState(), lldb.eStateExited, PROCESS_EXITED)
